#ifndef _MDVDP_INI_FILE_READER_
#define _MDVDP_INI_FILE_READER_

#include "Page.h"
#include "Selection.h"

// TODO : keep current line number to report error
// TODO : do more error checking on data 

int __fastcall as2i(AnsiString str)
{
	int result;

	try
	{
		result = str.ToInt();
	}
	catch(...)
	{
		result = 0;
	}

	return result;
}

//---------------------------------------------------------------------------
class   MDVDPFileException
{
public:
  MDVDPFileException (AnsiString msg):msg (msg)
	{
  }

  AnsiString msg;
};

//---------------------------------------------------------------------------
class   LineSection
{
public:
  LineSection (AnsiString line)
  {
		int posEqual = line.Pos ("=");

		key = line.SubString (1, posEqual - 1);
    params = line.SubString (posEqual + 1, line.Length () - posEqual);
	}

  AnsiString key;
  AnsiString params;
};

//---------------------------------------------------------------------------
class PageSelectionLine
{
public:
	PageSelectionLine(AnsiString line)
	{
		int     posEq = line.Pos ("=");
		int     posCB = line.Pos ("]");
		int     posSC = line.Pos (";");

		type = line.SubString (2, posEq - 2);
		num = line.SubString (posEq + 1, posCB - posEq - 1);
		if (posSC != 0)
			comment = line.SubString (posSC + 1, line.Length () - posSC).Trim ();
		else
			comment = "";
	}
	AnsiString type;
	AnsiString num;
	AnsiString comment;
};
//---------------------------------------------------------------------------

typedef void __fastcall (__closure *TOnNewPageRead)(Page* );
typedef void __fastcall (__closure *TOnNewSelectionRead)(Selection* ,Page* );

//---------------------------------------------------------------------------

class MDVDPIniFileReader {
public:
	MDVDPIniFileReader(AnsiString name)
	{
		try
		{
			fs = new TFileStream(name, fmOpenRead | fmShareDenyWrite);
			LineNumber = 1;
		} catch(...) {
			throw MDVDPFileException("Can't open file");
		}
		StartPage = -1;
	}

	~MDVDPIniFileReader()
	{
		delete fs;
	}

	void ParseBeginning()
	{
		if( GotoSection("[MENU]") )
		{
			if( ! ParseMenuSection() )
			{
				throw MDVDPFileException("Menu section must contain at least, Directory, AVIName & StartPage");
			}
		} else {
			throw MDVDPFileException("Can't find menu section");
		}
	}

	void ParsePagesAndSelections()
	{
		ParsePages();
	}

	// To get list of files for languages & subtitles
	bool ParseFileList(AnsiString section, TStringList* lst)
	{
		bool res = false;
		if( GotoSection(section) )
		{
			// Read each line to the end of the file or to the next section
			while( ReadLine() != -1 && (Line == "" || Line[1] != '[') )
			{
				// Parse a menu section line
				LineSection ls(Line);
				if(ls.key == "File")
				{
					lst->Add( ls.params );
					res = true;
				}
				else if(lst->Count >0 && (ls.key == "Format" || ls.key == "OpenBracket" ||
								ls.key == "CloseBracket" || ls.key == "LineSeparator" ||
								ls.key == "Offset" || ls.key == "Multiplier") )
				{
					// Read key associated with the last file
					TStringList *l;
					l = (TStringList*)lst->Objects[lst->Count-1];
					if(l == NULL)
					{
						l = new TStringList();
						lst->Objects[lst->Count-1] = l;
					}
					l->Add( ls.key + "=" + ls.params );
				}
			}
		}
		return res;
	}

private:
	int ReadLine()
	{
		char c;
		Line = "";
		int byteRead = 0;
		// Jump empty lines
		while( (byteRead = fs->Read(&c,1)) == 1 && ( c == '\r' || c == '\n' ) )
		{
			if(c == '\n')
				LineNumber++;
		}

		do
		{
			Line += c;
		} while ( (fs->Read(&c,1) == 1) && (c != '\r') );

		Line = Line.Trim();
		if(Line == NULL)
			Line = "";

		return (c == '\r') ? Line.Length() : -1;
	}

	bool GotoSection(AnsiString s) {
		// go to beginning
		LineNumber = 1;
		fs->Seek(0,soFromBeginning);

		bool find = false;
		// Search for section
		while( ReadLine() > 0 )
		{
			if( Line.UpperCase().Pos(s) != 0 )
			{
				find = true;
				break;
			}
		}
		return find;
	}

	bool ParseMenuSection() {
		// Read each line to the end of the file or to the next section
		while( ReadLine() != -1 && (Line == "" || Line[1] != '[') )
		{
			// Parse a menu section line
			LineSection ls(Line);

			if(ls.key == "AVIName")
			{
				AVIName = ls.params;
			} else if(ls.key == "Directory") {
				Directory = ls.params;
			} else if(ls.key == "StartPage") {
				StartPage = as2i(ls.params);
			}
		}
		return ( !AVIName.IsEmpty() && !Directory.IsEmpty() && StartPage > 0 );
	}

	void ParsePages()
	{
		// go to beginning
		LineNumber = 1;
		fs->Seek(0,soFromBeginning);

		// Go to first page
		if( GotoSection("[PAGE=") ) {
			// Current buffer contain first definition page [PAGE=x]    ; blah
			PageSelectionLine psl(Line);
			ParseOnePage(as2i(psl.num), psl.comment);
		}
	}

	void ParseOnePage(int number, AnsiString Comment)
	{
		OutputDebugString(  (AnsiString("ParseOnePage(")+number+","+Comment+")").c_str() );

		if( Comment.IsEmpty() )
		{
			Comment = "Untitled";
		}
		Page* p = new Page(number, Comment,NULL);

		while( ReadLine() != -1 && (Line == "" || Line[1] != '[') )
		{
			LineSection ls(Line);

			if(ls.key == "BackgroundMode")
			{
				if(ls.params == "1" || ls.params == "2")
				{
					p->isVideoType = ((ls.params == "1") ? true : false);
				}	else {
					ShowMessage( AnsiString().sprintf("Syntax error on line %d <%s>" , LineNumber, Line.c_str()) );
				}
			} else if(ls.key == "PictureFile") {
				p->PictureFileName = ls.params;
			} else if(ls.key == "VideoRange") {
				int ff,lf;
				if( sscanf(ls.params.c_str(),"%d-%d",&ff,&lf) )
				{
					p->FirstFrameNumber = ff;
					p->LastFrameNumber = lf;
				} else {
					ShowMessage( AnsiString().sprintf("Syntax error on line %d <%s>" , LineNumber, Line.c_str()) );
				}
			} else if(ls.key == "VideoLoop") {
				if(ls.params == "0" || ls.params == "1")
				{
					p->VideoLoop = ( (ls.params == "1") ? true : false );
				}	else {
					ShowMessage( AnsiString().sprintf("Syntax error on line %d <%s>" , LineNumber, Line.c_str()) );
				}
			} else if(ls.key == "PictureDuration") {
				p->PictureDuration = as2i(ls.params);
			} else if(ls.key == "JumpAtEnd") {
				p->JumpAtEnd = as2i(ls.params);
			} else if(ls.key == "ActionAtEnd") {
				OutputDebugString(  ls.params.c_str() );			
				// ActionAtEnd=[1-8]?#
				int actionType = ls.params[1] - '0';
				MenuAction* a = new MenuAction( actionType, ls.params.SubString( 3, ls.params.Length() ) );
				p->ActionAtEnd = a;
			} else if(ls.key == "SelectionAvailable") {
				p->SelectionAvailable = (ls.params == "1");
			} else if(ls.key == "DefaultSelection") {
				p->DefaultSelection = as2i(ls.params);
			} else if(ls.key == "ProhibitSelectionBefore") {
				p->ProhibitSelectionBefore = as2i(ls.params);
			} else if(ls.key == "ProhibitSelectionAfter") {
				p->ProhibitSelectionAfter = as2i(ls.params);
			} else if(ls.key == "SelectionOrdering") {
				p->ColumnOrdering = (ls.params == "2");
			}
		}

		// Create new page
		if(FOnNewPageRead != NULL) {
			FOnNewPageRead(p);
		}

		// Process next section, page or selection ?
		PageSelectionLine psl(Line);
		if ( psl.type.AnsiCompareIC("page") == 0 )
		{
			ParseOnePage(as2i(psl.num), psl.comment);
		} else if ( psl.type.AnsiCompareIC("selection") == 0 ) {
			ParseOneSelection(as2i(psl.num), psl.comment, p);
		}
	}

	void ParseOneSelection(int number, AnsiString Comment, Page* parentPage) {
		OutputDebugString(  (AnsiString("ParseOneSelection(")+number+","+Comment+")").c_str() );

		if( Comment.IsEmpty() )
		{
			Comment = "Untitled";
		}
		Selection* s = new Selection("", Comment, NULL);

		while( ReadLine() != -1 && (Line == "" || Line[1] != '[') )
		{
			LineSection ls(Line);

			if(ls.key == "CopyFrom")
			{
				// TODO : Support CopyFrom
			} else if(ls.key == "Area") {
				s->Area = ls.params;
			} else if(ls.key == "Action") {
				// action list
				// ex: Action=1
				// ex: Action=1#2#3
				for( int i = 0; i < ls.params.Length(); i += 2 ) {
					int actionType = (ls.params[i+1] - '0');
					if( actionType > 0 && actionType <= 8)
					{
						MenuAction* a = new MenuAction(actionType,"");
						s->vectAction.push_back( a );
					} else {
						ShowMessage( AnsiString().sprintf("Syntax error on line %d <%s>" , LineNumber, Line.c_str()) );
					}
				}
			} else if(ls.key == "Parameter") {
				// retrieve parameters list
				// ex : Parameter=12#1#Text Message#123#+3
				AnsiString token;
				int pos;
				for(unsigned int i=0; i < s->vectAction.size(); i++) {
					switch(s->vectAction[i]->type) {
						case 1:	// page number -> 1..500 or +1..? or -1..?
							// Support +1 -1
							pos = ls.params.Pos("#");
							if( pos > 0 )
							{
								// Multiple parameters
								if(ls.params[1] == '+' || ls.params[1] == '-')
								{
									int offset = as2i(ls.params.SubString(1,pos-1));
									if(offset == 1 || offset == -1)
									{
										s->vectAction[i]->type = (RELATIVE_JUMP+1);
										s->vectAction[i]->param = AnsiString(offset);
									} else {
										// TODO : Check if we need to jump to page number + offset
										// or jump to offset page after the current page ???
										int pageNumber = parentPage->Number +	offset;
										s->vectAction[i]->param = AnsiString(pageNumber);
									}
								} else {
									ls.params = ls.params.SubString(pos+1,ls.params.Length());
								}
							} else if(ls.params.Length() > 0) {
								// One parameter
								if(ls.params[1] == '+' || ls.params[1] == '-')
								{
									int offset = as2i(ls.params);
									if(offset == 1 || offset == -1)
									{
										s->vectAction[i]->type = (RELATIVE_JUMP+1);
										s->vectAction[i]->param = AnsiString(offset);
									} else {
										// TODO : Check if we need to jump to page number + offset
										// or jump to offset page after the current page ???
										s->vectAction[i]->param = AnsiString( parentPage->Number +
											offset );
									}
								} else {
									s->vectAction[i]->param = ls.params;
								}
							}
							break;
						case 2:
						case 8:
							// no parameters
							break;
						case 3:  // chapter
						case 5:  // special
						case 6:	 // language
						case 7:  // subtitle
							pos = ls.params.Pos("#");
							if( pos > 0 ) {
								s->vectAction[i]->param = ls.params.SubString(1,pos-1);
								ls.params = ls.params.SubString(pos+1,ls.params.Length());
							} else if(ls.params.Length() > 0) {
								s->vectAction[i]->param = ls.params;
							}
							break;
						case 4:
							s->vectAction[i]->param = ls.params;
							break;
						default:
							break;
					}
				}
			}
		}
		// Create new selection
		if(FOnNewSelectionRead != NULL)
		FOnNewSelectionRead(s, parentPage);

		// Process next section, page or selection ?
		PageSelectionLine psl(Line);
		if ( psl.type.AnsiCompareIC("page") == 0 )
		{
			ParseOnePage(as2i(psl.num), psl.comment);
		} else if ( psl.type.AnsiCompareIC("selection") == 0 ) {
			ParseOneSelection(as2i(psl.num), psl.comment, parentPage);
		}
	}

private:
	long LineNumber;
	AnsiString Line;
	TFileStream *fs;
	TOnNewPageRead FOnNewPageRead;
	TOnNewSelectionRead FOnNewSelectionRead;

public:
		__property TOnNewPageRead OnNewPageRead = { read = FOnNewPageRead, write = FOnNewPageRead };
		__property TOnNewSelectionRead OnNewSelectionRead = { read = FOnNewSelectionRead, write = FOnNewSelectionRead };

	AnsiString AVIName;
	AnsiString Directory;
	int StartPage;
};

//---------------------------------------------------------------------------

#endif

