/***************************************************************************\
 * Micro DVD Menu Maker by Christophe.Paris@free.fr                        *
 ***************************************************************************
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
\***************************************************************************/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "PageViewer.h"
#include "SelectionViewer.h"
#include "MDVDPIniFileReader.h"
#include "MovieData.h"
#include "About.h"

#define CURSOR_CROSS_CENTER_SPACE 15

#define MIN_MAINFORM_WIDTH		440
#define MIN_MAINFORM_HEIGHT		426

#include <shlobj.h>
#include <vfw.h>
#include <inifiles.hpp>
#include <registry.hpp>

#define CLIP_ENABLE

TColor ColorOK  = (TColor)0x00E6FFE6;
TColor ColorNOK = (TColor)0x00E6E6FF;

//---------------------------------------------------------------------------

// TODO : BUGFIX - With OnMouseDown event, on page form you must click 2 time to show selection
// TODO : FEATURE - Import a full folder of still image
// TODO : FEATURE - Check if relative jumps are OK before saving
// TODO : FEATURE - Add shortcuts (added Save = Ctrl-S)
// TODO : FEATURE - Add MRU
// TODO : FEATURE - Multi-language support
// TODO : FEATURE - Make a copy/replace selection option
// TODO : FEATURE - Allow page number change
// TODO : FEATURE - Do a check on video & Still image size (must be the same)
// TODO : FEATURE - Save position of forms & if they are docked
// TODO : FEATURE - Find a way to have a page loop preview with sound (eg. play the page range in a media player)
// TODO : FEATURE - Magnetic grid with variable size
// TODO : FEATURE - Zoom x0.5 (for big pictures/video & small screen)
// TODO : FEATURE - Resize all selections (usefull if you change the size of your menu)
// TODO : FEATURE - Read video frame number in graph avi

// DONE : FEATURE - Save last base directory used & start in this directory
// FIXED : BUGFIX - ActionAtEnd & JumpAtEnd are now allowed on still image page (don't forget to use it with PictureDuration > 0)
// FIXED : BUGFIX - When saving check if there is at least 1 movie file
// DONE : FEATURE - For still image page, page name = JPEG file name.
// DONE : FEATURE - Allow menu action modification
// DONE : FEATURE - Make intelligent copy ( if copy chapters, auto increment numbers ...)
// DONE : FEATURE - Add a pop up menu to action at end (for docked mode)
// DONE : FEATURE - Dockable tools windows (you can use Ctrl-Up & Ctrl-Down to change pages)
// FIXED : BUGFIX - Creation of page take range in text field and not have always CurrentPos-0
// FIXED : BUGFIX - Tab order has been redone
// FIXED : BUGFIX - Parameter check box is bigger (it display up to 32 items without scrolling)
// DONE : FEATURE - +1 -1 in page links
// FIXED : BUGFIX - AVI Video files filter now include fake vob avi

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;

PAVIFILE pAvifile;
PAVISTREAM pAviStream;
AVISTREAMINFO videoStreamInfo;
PGETFRAME pGetFrame;

bool isDrawing;
bool isSelectingMoving;
int origX, origY;
RShape *currentShape;
RShape *shapeModel;
bool crossCursorIsVisible = true;

AnsiString tmpVideoPos;
bool ShuttingDown = false;

RECT oldClipRect;

const AnsiString NewFileName = "New File";

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
	LoadOptions();

	CurrentFileName = NewFileName;
	UpdateCaption();

	Application->HintPause = 100;
	Application->HintHidePause = 30000;

	GetClipCursor(&oldClipRect);

	shapeModel = new RShape(VideoPanel);
	shapeModel->Visible = false;
	shapeModel->Parent = VideoPanel;
	shapeModel->OnContextPopup = ShapeContextPopup;
	shapeModel->OnChangedSize = OnCurrentSelectionChangedSize;
	shapeModel->OnAnyAnchorMove = OnAnyAnchorMoveHideCrossCursor;
	shapeModel->OnMouseDown = ShapeMouseDown;
	shapeModel->OnMouseMove = ShapeMouseMove;
	shapeModel->OnMouseUp = ShapeMouseUp;
	shapeModel->Pen->Color = clWhite;
	shapeModel->Pen->Mode = pmXor;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miMMQuitClick(TObject *Sender)
{
	PostQuitMessage(0);
}

//---------------------------------------------------------------------------

bool ShowAVIFrame(unsigned int i)
{
	if(pGetFrame == NULL)
		return false;

	LPBITMAPINFOHEADER lpbi;
	if( (lpbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pGetFrame,i)) == NULL)
	{
		MessageBox(PageViewerForm->Handle, "AVIStreamGetFrame FAILED!!!!", "Error",
			MB_OK|MB_ICONERROR);
		return false;
	}
	else
	{
		// Write the bitmap in a TStream

		// The packed DIB is comprised of a BITMAPINFOHEADER structure, followed
		// by the palette information if needed, and then followed by the bitmap
		// data. All of this is just one sequential block of memory, so its
		// possible to calculate the palette and bitmap pointers using pointer
		// arithmetic.

		// TODO : check if 8 bits
		TMemoryStream* fs = new TMemoryStream();

    // MainForm->Memo1->Lines->Add("biWidth="+AnsiString( lpbi->biWidth) );
    // MainForm->Memo1->Lines->Add("biHeight="+AnsiString( lpbi->biHeight) );
    // MainForm->Memo1->Lines->Add("biBitCount="+AnsiString( lpbi->biBitCount) );

		DWORD size = sizeof(BITMAPINFOHEADER);
		size += (lpbi->biWidth * lpbi->biBitCount * lpbi->biHeight) / 8;

    // MainForm->Memo1->Lines->Add("size = biWidth * biHeight * biBitCount = "+AnsiString( size) );

		// make the bitmap header
		BITMAPFILEHEADER bfHeader;
		bfHeader.bfType = ((WORD) ('M' << 8) | 'B');  // 'BM'
		bfHeader.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER)
											 + (DWORD) sizeof(BITMAPINFOHEADER);
		bfHeader.bfSize = sizeof(BITMAPFILEHEADER) + size;
		bfHeader.bfReserved1 = bfHeader.bfReserved2 = 0;


		fs->Write(&bfHeader,sizeof(BITMAPFILEHEADER));
    // MainForm->Memo1->Lines->Add("Writing header ("+AnsiString( sizeof(BITMAPFILEHEADER) ) + ")");
		fs->Write(lpbi,size);
    // MainForm->Memo1->Lines->Add("Writing data ("+AnsiString( size ) + ")");
    // MainForm->Memo1->Lines->Add(AnsiString(res) + " byte written");

		fs->Seek(0,0);
		MainForm->VideoImage->Picture->Bitmap->LoadFromStream(fs);

		delete fs;
	}
	return true;
}

//---------------------------------------------------------------------------
// Open an AVI file & the first video stream
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenAVI(AnsiString filename)
{
	// Look if an AVI is already open
	if(pAvifile != NULL)
	{
		// Free old file ressources
		AVIFileRelease(pAvifile);  // closes the file
		AVIFileExit();          // releases AVIFile library
	}

	AVIFileInit();          // opens AVIFile library

	if( AVIFileOpen(&pAvifile, filename.c_str(),
			OF_SHARE_DENY_WRITE, NULL) != AVIERR_OK )
	{
		MessageBox(PageViewerForm->Handle, "Unable to open AVI file", "Error", MB_OK|MB_ICONERROR);
		return;
	}

	// Open the first Video stream
	if( AVIFileGetStream(pAvifile, &pAviStream,
			streamtypeVIDEO, 0) != AVIERR_OK )
	{
		MessageBox(PageViewerForm->Handle, "Unable to open the video stream", "Error",
			MB_OK | MB_ICONERROR);
		// Free avi file
		AVIFileRelease(pAvifile);  // closes the file
		AVIFileExit();          // releases AVIFile library
		return;
	}

	if( AVIStreamInfo(pAviStream, &videoStreamInfo, sizeof(AVISTREAMINFO)) != 0 )
	{
		MessageBox(PageViewerForm->Handle, "Can't get video stream information", "Warning",
			MB_OK | MB_ICONWARNING);
	}

	pGetFrame = AVIStreamGetFrameOpen(pAviStream, NULL);

	// ALL IS OK
	VideoTrackBar->Max = videoStreamInfo.dwLength - 1;
	if(VideoTrackBar->Position != 0)
	{
			VideoTrackBar->Position = 0;
	}
	else
	{
			ShowAVIFrame(0);
	}

	ResizeVideoImage(videoStreamInfo.rcFrame.right,
		videoStreamInfo.rcFrame.bottom);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ResizeVideoImage(int width, int height)
{
	// Resize Form so VideoImage horizontally fit
	if( width + 24 < MIN_MAINFORM_WIDTH )
	{
		MainForm->Width = MIN_MAINFORM_WIDTH;
		// The Form can't be smallest so center the video
		VideoImage->Left = (VideoPanel->Width - width) / 2;
	}
	else
	{
		VideoImage->Left = 8;
		MainForm->Width = width + 24;
	}

	// Resize Form so VideoImage vertically fit
	if( height + VideoTrackBar->Height +
			ControlPanel->Height + 62 < MIN_MAINFORM_HEIGHT )
	{
		MainForm->Height = MIN_MAINFORM_HEIGHT;
		// The form can't be smallest so center the video
		VideoImage->Top = (VideoPanel->Height - VideoTrackBar->Height - height) / 2;
	}
	else
	{
		VideoImage->Top = 8;
		MainForm->Height = height + VideoTrackBar->Height +
			ControlPanel->Height + 62;
	}

	VideoBevel->Top = MainForm->VideoImage->Top - 2;
	VideoBevel->Left = MainForm->VideoImage->Left - 2;
	VideoBevel->Width = MainForm->VideoImage->Width + 4;
	VideoBevel->Height = MainForm->VideoImage->Height + 4;

	HLine1->Left = VideoImage->Left;
	VLine1->Top = VideoImage->Top;

	// Put subform around MainForm
	OnMainFormMove(TMessage());
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
	ShuttingDown = true;

	delete shapeModel;

	// Look if an AVI is already open
	if(pAvifile != NULL)
	{
		// Free file ressources
		AVIStreamGetFrameClose(pGetFrame);
		AVIStreamRelease(pAviStream);
		AVIFileRelease(pAvifile);  // closes the file
		AVIFileExit();          // releases AVIFile library
	}

	SaveOptions();	
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::VideoTrackBarChange(TObject *Sender)
{
	static unsigned char hh, mm, ss, ms;
	static unsigned long TimeInMs;
	static bool isKF;

	if (pAvifile == NULL)
	{
		return;
	}

  edVideoPos->Text = VideoTrackBar->Position;
  ShowAVIFrame (VideoTrackBar->Position);
  // Display time
  TimeInMs = AVIStreamSampleToTime (pAviStream, VideoTrackBar->Position);

  hh = TimeInMs / (3600 * 1000);
  mm = (TimeInMs - (hh * 3600 * 1000)) / (60 * 1000);
  ss = (TimeInMs - (hh * 3600 * 1000) - (mm * 60 * 1000)) / 1000;
  ms = (TimeInMs - (hh * 3600 * 1000) - (mm * 60 * 1000) - (ss * 1000));

  // Is the current frame a KF ?
	isKF = (AVIStreamFindSample (pAviStream,
		VideoTrackBar->Position, FIND_KEY) == VideoTrackBar->Position);

	FrameInfoPanel->Caption =
		AnsiString ().sprintf ("  Frame %d (%d:%02d:%02d.%03d) [%c]",
				 VideoTrackBar->Position, hh, mm, ss, ms, isKF ? 'K' : ' ');

	// Make page range & frame number match

	// First check with current page
  /*
	bool pageOK = false;
	Page* p;
	if( PageViewerForm->PageList->ItemIndex != -1)
	{
		p = (Page*) PageViewerForm->PageList->Items->Objects[ PageViewerForm->PageList->ItemIndex ];
		if( VideoTrackBar->Position >= p->FirstFrameNumber &&
				VideoTrackBar->Position <= p->LastFrameNumber )
		{
			// OK
			pageOK = true;
		}
	}
	if( !pageOK ) {
		// Search if there is a corresponding page
		bool pageFound = false;
		for(int i = 0; i < PageViewerForm->PageList->Items->Count; i++)
		{
			p = (Page*) PageViewerForm->PageList->Items->Objects[ i ];
			if( VideoTrackBar->Position >= p->FirstFrameNumber &&
					VideoTrackBar->Position <= p->LastFrameNumber )
			{
					PageViewerForm->PageList->ItemIndex = i;
					PageViewerForm->ShowPage( PageViewerForm->PageList->ItemIndex,
						false );
					pageFound = true;
					break;
			}
		}
		// if frame is not associated with a page hide selections
		if( !pageFound && PageViewerForm->PageList->ItemIndex != -1)
		{
			PageViewerForm->PageList->ItemIndex = -1;
			PageViewerForm->ShowPage( PageViewerForm->PageList->ItemIndex,
				false);
		}
	}
  */
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttNextFrameClick(TObject *Sender)
{
	VideoTrackBar->Position++;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttPrevFrameClick(TObject *Sender)
{
	VideoTrackBar->Position--;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttEndClick(TObject *Sender)
{
	VideoTrackBar->Position = VideoTrackBar->Max;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttStartClick(TObject *Sender)
{
	VideoTrackBar->Position = VideoTrackBar->Min;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttNextKFClick(TObject *Sender)
{
	static long nextKF;

	if(pAviStream == NULL)
	{
		return;
	}

	nextKF = AVIStreamFindSample(pAviStream, VideoTrackBar->Position + 1,
		FIND_KEY | FIND_NEXT);

	if(nextKF > VideoTrackBar->Position)
	{
		VideoTrackBar->Position = nextKF;
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::bttPrevKFClick(TObject *Sender)
{
	if(pAviStream == NULL)
	{
		return;
	}

	VideoTrackBar->Position = AVIStreamFindSample(pAviStream,
		VideoTrackBar->Position - 1, FIND_KEY | FIND_PREV);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::MenuItemNewPageClick(TObject *Sender)
{
	PageViewerForm->bttAdd->Click();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::MenuItemDeletePageClick(TObject *Sender)
{
	PageViewerForm->bttDel->Click();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::MenuItemModifyNamePageClick(TObject *Sender)
{
	PageViewerForm->bttModify->Click();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::MenuItemFirstFramePageClick(TObject *Sender)
{
	PageViewerForm->bttFirst->Click();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::MenuItemLastFramePageClick(TObject *Sender)
{
	PageViewerForm->bttLast->Click();
}

//---------------------------------------------------------------------------
//===========================================================================
//
// Start of selection
void __fastcall TMainForm::VideoImageMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbLeft)
	{
		if(currentShape != NULL)
		{
			currentShape->ShowAnchors(false);
		}

		currentShape = new RShape(VideoPanel);
		currentShape->Width = 0;
		currentShape->Height = 0;
		currentShape->Parent = VideoPanel;
		currentShape->OnContextPopup = ShapeContextPopup;
		currentShape->OnChangedSize = OnCurrentSelectionChangedSize;
		currentShape->OnAnyAnchorMove = OnAnyAnchorMoveHideCrossCursor;

		currentShape->OnMouseDown = ShapeMouseDown;
		currentShape->OnMouseMove = ShapeMouseMove;
		currentShape->OnMouseUp = ShapeMouseUp;

		currentShape->Pen->Color = clWhite;
		currentShape->Pen->Mode = pmXor;

		currentShape->Top = Y + VideoImage->Top;
		currentShape->Left = X + VideoImage->Left;
		isDrawing = true;
	}

	GetClipCursor(&oldClipRect);
	TPoint p(X,Y);
	p = VideoImage->ClientToScreen(p);
	RECT newClipRect;
	newClipRect.left = p.x;
	newClipRect.top = p.y;
	newClipRect.right = p.x + (VideoImage->Width - X);
	newClipRect.bottom = p.y + (VideoImage->Height - Y);

#ifdef CLIP_ENABLE
	ClipCursor(&newClipRect);
#endif
}

//---------------------------------------------------------------------------
// Make the selection grow while the mouse move
//---------------------------------------------------------------------------
void __fastcall TMainForm::VideoImageMouseMove(TObject *Sender,
	TShiftState Shift, int X, int Y)
{
	if(! crossCursorIsVisible)
	{
		ShowCrossCursor(true);
	}

	if(Shift.Contains(ssLeft) && isDrawing)
	{
		currentShape->Width = (X + VideoImage->Left) - currentShape->Left;
		currentShape->Height = (Y + VideoImage->Top) - currentShape->Top;
	}

	HLine1->Top = Y + VideoImage->Top;
	HLine1->Width = X - CURSOR_CROSS_CENTER_SPACE;

	HLine2->Top = Y + VideoImage->Top;
	HLine2->Left = X + VideoImage->Left + CURSOR_CROSS_CENTER_SPACE;
	HLine2->Width = VideoImage->Width - X - CURSOR_CROSS_CENTER_SPACE;

	VLine1->Left = X + VideoImage->Left;
	VLine1->Height = Y - CURSOR_CROSS_CENTER_SPACE;

	VLine2->Left = X + VideoImage->Left;
	VLine2->Top = Y + VideoImage->Top + CURSOR_CROSS_CENTER_SPACE;
	VLine2->Height = VideoImage->Height - Y - CURSOR_CROSS_CENTER_SPACE;
}

//---------------------------------------------------------------------------
// Ending selection
//---------------------------------------------------------------------------
void __fastcall TMainForm::VideoImageMouseUp(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbLeft && isDrawing)
	{
		// Really add the shape if it's big enough
		currentShape->Width = (X + VideoImage->Left) - currentShape->Left;
		currentShape->Height = (Y + VideoImage->Top) - currentShape->Top;
		if(currentShape->Width < 5 || currentShape->Height < 5 || PageViewerForm->PageList->ItemIndex == -1)
		{
			delete currentShape;
			currentShape = NULL;
		}
		else
		{
			// Add the shape to the current page
			Page* p = (Page*) PageViewerForm->PageList->Items->Objects[
				PageViewerForm->PageList->ItemIndex ];
			Selection* sel = new Selection( GetCurrentShapeXYXYCoordinate(),
				AnsiString("Selection-") + (++p->SelectionNumber), currentShape );
			currentShape->Tag = (int) sel;
			p->vectSel.push_back( sel );
			SelectionsViewerForm->ShowPageSelections(p);
			PageViewerForm->UpdateDefaultSelectionCombo(p);			
		}
	}
	isDrawing = false;

#ifdef CLIP_ENABLE
	ClipCursor(&oldClipRect);
#endif
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShapeMouseDown(TObject *Sender, TMouseButton Button,
	TShiftState Shift, int X, int Y)
{
	// The shape will be moved and selected
	if(Button == mbLeft)
	{
		// Deselect currentshape
		if(currentShape != NULL)
		{
			currentShape->ShowAnchors(false);
		}

		currentShape = static_cast<RShape*>(Sender);
		currentShape->BringToFront();
		isSelectingMoving = true;
		// Remember current coord of the mouse
		origX = X;
		origY = Y;

		// Limit cursor move
		GetClipCursor(&oldClipRect);
		TPoint p(0,0);
		p = VideoImage->ClientToScreen(p);
		RECT newClipRect;
		newClipRect.left = p.x + X;		newClipRect.top = p.y + Y;
		newClipRect.right = p.x + VideoImage->Width - (currentShape->Width - X);
		newClipRect.bottom = p.y + VideoImage->Height - (currentShape->Height - Y);;

#ifdef CLIP_ENABLE
		ClipCursor(&newClipRect);
#endif
	}
}

//---------------------------------------------------------------------------
// Selection pop up
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShapeContextPopup(TObject *Sender,
	const TPoint &MousePos, bool &Handled)
{
	if(Sender->ClassNameIs("RShape"))
	{
		// Deselect currentshape
		if(currentShape != NULL)
		{
			currentShape->ShowAnchors(false);
		}
		currentShape = static_cast<RShape*>(Sender);
		TPoint p1 = currentShape->ClientToScreen(MousePos);
		// Complete pop up menu with page list
		Movetopage1->Enabled = (PageViewerForm->PageList->Items->Count > 1);
		if(Movetopage1->Enabled)
		{
			Page* p;
			Movetopage1->Clear();
			for(int i=0; i < PageViewerForm->PageList->Items->Count; i++)
			{
				TMenuItem *mi = new TMenuItem(Movetopage1);
				p = (Page*) PageViewerForm->PageList->Items->Objects[ i ];
				mi->Caption = (AnsiString(p->Number) + ") " + p->Name);
				mi->RadioItem = true;
				mi->OnClick = OnChangePageSelectionClick;
				mi->Tag = (int)p;  // Keep a pointer to the destination page
				if(i == PageViewerForm->PageList->ItemIndex )
				{
					mi->Checked = true;
					mi->Enabled = false;
				}
				Movetopage1->Add(mi);
			}
		}
		Handled = true;
		ShapePopUp->Popup(p1.x,p1.y);
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::Delete1Click(TObject *Sender)
{
	if(PageViewerForm->PageList->ItemIndex >= 0)
	{
		// Search the current selection in the page
		Page* p = (Page*)PageViewerForm->PageList->Items->Objects[
			PageViewerForm->PageList->ItemIndex ];
		Selection* s = (Selection*)currentShape->Tag;
		vector<Selection*>::iterator si = find(p->vectSel.begin(),
			p->vectSel.end(), s);
		p->vectSel.erase( si );
		delete s;
		SelectionsViewerForm->ShowPageSelections(p);
		PageViewerForm->UpdateDefaultSelectionCombo(p);
	}
	else
	{
		delete currentShape;
	}
	currentShape = NULL;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShapeMouseUp(TObject *Sender, TMouseButton Button,
	TShiftState Shift, int X, int Y)
{
	// Select the shape
	// make 8 shape to allow selection modifications
	if(Button == mbLeft && isSelectingMoving)
	{
		currentShape->ShowAnchors(true);
		// Select it in SelectionViewer
		Selection* s = (Selection*)currentShape->Tag;
		SelectionsViewerForm->SelectSelection(s);
		s->Area = GetCurrentShapeXYXYCoordinate();
#ifdef CLIP_ENABLE
		ClipCursor(&oldClipRect);
#endif
	}
	isSelectingMoving = false;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShapeMouseMove(TObject *Sender, TShiftState Shift,
	int X, int Y) {

	if(crossCursorIsVisible) {
		ShowCrossCursor(false);
	}

	// Move the shape
	if(Shift.Contains(ssLeft) && isSelectingMoving)
	{
		// Move the shape
		currentShape->Top += (Y - origY);
		currentShape->Left += (X - origX);
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::OnCurrentSelectionChangedSize(void)
{
	Selection* s = (Selection*)currentShape->Tag;
	// Update selection area
	s->Area = GetCurrentShapeXYXYCoordinate();
}

//===========================================================================
//---------------------------------------------------------------------------
void __fastcall TMainForm::bttGotoClick(TObject *Sender)
{
	if(!edVideoPos->Text.IsEmpty())
	{
		try
		{
			VideoTrackBar->Position = edVideoPos->Text.ToInt();
		} catch(...) {
			edVideoPos->Text = 0;
			VideoTrackBar->Position = 0; 
		}
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::edVideoPosEnter(TObject *Sender)
{
	tmpVideoPos = edVideoPos->Text;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::edVideoPosExit(TObject *Sender)
{
	if(edVideoPos->Text.IsEmpty())
	{
		edVideoPos->Text = tmpVideoPos;
	}
	else
	{
		try
		{
			edVideoPos->Text.ToInt();
		}
		catch(...)
		{
			edVideoPos->Text = tmpVideoPos;
		}
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::OnMainFormMove(TMessage &Message)
{
	if(ShuttingDown || PageViewerForm == NULL || SelectionsViewerForm == NULL)
		return;

	if(miDocked->Checked)
		return;

	if(!miWinPagesFree->Checked)
	{
		PageViewerForm->Top  = MainForm->Top;
	}

	if(!miWinSelecFree->Checked)
	{
		SelectionsViewerForm->Top  = MainForm->Top;
	}

	if(miWinPagesLeft->Checked)
	{
		if(!miWinPagesFree->Checked)
		{
			PageViewerForm->Left = MainForm->Left - PageViewerForm->Width;
		}

		if(miWinSelecLeft->Checked)
		{
			/* S | P | M */
			if(!miWinSelecFree->Checked)
			{
				SelectionsViewerForm->Left = MainForm->Left -
					PageViewerForm->Width - SelectionsViewerForm->Width;
			}
		}
		else
		{
			/* P | M | S*/
			if(!miWinSelecFree->Checked)
			{
				SelectionsViewerForm->Left = MainForm->Left + MainForm->Width;
			}
		}
	}
	else
	{
		if(!miWinPagesFree->Checked)
		{
			PageViewerForm->Left = MainForm->Left + MainForm->Width;
		}
		if(miWinSelecLeft->Checked)
		{
			/* S | M | P */
			if(!miWinSelecFree->Checked)
			{
				SelectionsViewerForm->Left = MainForm->Left -
					SelectionsViewerForm->Width;
			}
		}
		else
		{
			/* M | P | S */
			if(!miWinSelecFree->Checked)
			{
				SelectionsViewerForm->Left = PageViewerForm->Left +
					PageViewerForm->Width;
			}
		}
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ChangeCurrentSelectedShape(RShape* r)
{
	if(currentShape != NULL)
	{
		currentShape->ShowAnchors(false);
	}
	currentShape = r;
	r->ShowAnchors(true);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShowCrossCursor(bool show)
{
	HLine1->Visible = show;
	HLine2->Visible = show;
	VLine1->Visible = show;
	VLine2->Visible = show;
	crossCursorIsVisible = show;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::OnAnyAnchorMoveHideCrossCursor(void)
{
	if(crossCursorIsVisible)
	{
		ShowCrossCursor(false);
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::RadioMenuItemCheck(TObject *Sender)
{
	TMenuItem* mi = dynamic_cast<TMenuItem*>(Sender);
	mi->Checked = true;
	OnMainFormMove(TMessage());
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::OnNewPageReadInINIFile(Page *p)
{
	// add a new page
	if( p->isVideoType )
	{
		ShowAVIFrame(p->FirstFrameNumber);
		VideoImage->Refresh();
	} else {
		TJPEGImage* JpegImg = new TJPEGImage();
		JpegImg->LoadFromFile(MovieDataForm->edMenuDirectory->Text +
			"\\" + p->PictureFileName);
		JpegImg->DIBNeeded();
		VideoImage->Picture->Bitmap->Assign(JpegImg);
		delete JpegImg; 
		MainForm->ResizeVideoImage(MainForm->VideoImage->Picture->Width,
			MainForm->VideoImage->Picture->Height);
		VideoImage->Refresh();
	}

	// Create a temporary small image
	TImage *smallImage = new TImage(this);
	// Specify new dimmension
	TRect DestRect;
	DestRect.Left = 0; DestRect.Top = 0;
	DestRect.Right = 64; DestRect.Bottom = 43;
	// Copy the image and resize it
	smallImage->Canvas->StretchDraw(DestRect, VideoImage->Picture->Bitmap);
	smallImage->Picture->Bitmap->Width = 64;
	smallImage->Picture->Bitmap->Height = 43;

	p->SetSmallImage(smallImage->Picture);

	// Free temporary small image
	delete smallImage;

	PageViewerForm->PageList->Items->AddObject(p->Number, (TObject*)p);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::OnNewSelectionReadInINIFile(Selection *s,
	Page* parentPage)
{
	// add a new Selection
	parentPage->vectSel.push_back( s );
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miMMOpenINIClick(TObject *Sender)
{
	// Open AVI file
	OpenDialog1->InitialDir = MovieDataForm->edBaseDirectory->Text;
	OpenDialog1->Filter = "MDVDP INI files (*.ini;*.mvd)|*.INI;*.MVD";
	if(OpenDialog1->Execute())
	{
		// Clear All
		PageViewerForm->ReInit();
		SelectionsViewerForm->ReInit();
		// To much things to reinit here so delete & recreate form
		delete MovieDataForm;
		MovieDataForm = new TMovieDataForm(this);

		CurrentFileName = OpenDialog1->FileName;
		UpdateCaption();
		MovieDataForm->edBaseDirectory->Text = OpenDialog1->FileName.SubString(1,
			OpenDialog1->FileName.LastDelimiter("\\"));
		miMovieData->Enabled = true;
		miPage->Enabled = true;
		miMMSaveasINI->Enabled = true;
		miMMSaveINI->Enabled = true;		

		TIniFile* pIniFile;
		try
		{
			// Read movie extra data like a normal INI File
			pIniFile = new TIniFile(OpenDialog1->FileName);

			// ----- Read [MAIN] section -----
			MovieDataForm->edMainTitle->Text = pIniFile->ReadString("MAIN",
				"Title", "");
			if( MovieDataForm->edMainTitle->Text.IsEmpty() )
			{
				ShowMessageMissINIKey("The key [MAIN]->Title is missing. Leaving...");
				MovieDataForm->edMainTitle->Color = ColorNOK;
				return;
			}
			MovieDataForm->edMainTitle->Color = ColorOK;
			MovieDataForm->edMainID->Text = pIniFile->ReadString("MAIN", "ID", "");
			if( MovieDataForm->edMainID->Text.IsEmpty() )
			{
				ShowMessageMissINIKey("The key [MAIN]->ID is missing. Leaving...");
				MovieDataForm->edMainID->Color = ColorNOK;
				return;
			}
			MovieDataForm->edMainID->Color = ColorOK;
			MovieDataForm->edMainCDNumber->Text = pIniFile->ReadString("MAIN",
				"CDNumber", "");
			MovieDataForm->edMainDelay->Text = pIniFile->ReadString("MAIN",
				"Delay", "");

			// ----- Read [MOVIE] section -----
			MovieDataForm->edMovieDirectory->Text = pIniFile->ReadString("MOVIE",
				"Directory", "");
			if( MovieDataForm->edMovieDirectory->Text.IsEmpty() )
			{
				ShowMessageMissINIKey("The key [MOVIE]->Directory is missing. Leaving...");
				return;
			}
			// TODO : check if directory exist
			MovieDataForm->edBaseDirectory->Color = ColorOK;

			MovieDataForm->edMovieAVIName->Text = pIniFile->ReadString("MOVIE",
				"AVIName", "");
			if( MovieDataForm->edMovieAVIName->Text.IsEmpty() )
			{
				ShowMessageMissINIKey("The key [MOVIE]->AVIName is missing. Leaving...");
				MovieDataForm->edMovieAVIName->Color = ColorNOK;				
				return;
			}
			MovieDataForm->edMovieAVIName->Color = ColorOK;
			MovieDataForm->edMovieAVI2Name->Text = pIniFile->ReadString("MOVIE",
				"AVI2Name", "");
			MovieDataForm->edMovieAVI3Name->Text = pIniFile->ReadString("MOVIE",
				"AVI3Name", "");
			MovieDataForm->edMovieCD1Frames->Text = pIniFile->ReadString("MOVIE",
				"CD1Frames", "");
			MovieDataForm->edMovieCD2Frames->Text = pIniFile->ReadString("MOVIE",
				"CD2Frames", "");
			MovieDataForm->edMovieCD3Frames->Text = pIniFile->ReadString("MOVIE",
				"CD3Frames", "");

			// ----- Read [CHAPTERS] section -----
			int i = 1, posSeparator;
			AnsiString strTmp;
			MovieDataForm->lstChapters->Items->Clear();
			while( (strTmp = pIniFile->ReadString("CHAPTERS", AnsiString(i), "")) != "" )
			{
				posSeparator = strTmp.Pos(" ");
				if( posSeparator > 0 )
				{
					// Add each chapter
					TListItem  *li = MovieDataForm->lstChapters->Items->Add();
					li->Caption = AnsiString(i);
					li->SubItems->Add( strTmp.SubString(1,posSeparator-1) );
					li->SubItems->Add( strTmp.SubString(posSeparator+1,strTmp.Length() -
						posSeparator) );
				}
				else
				{
					MessageBox(PageViewerForm->Handle, AnsiString("Malformed chapter entry : <" + strTmp +
						">, Leaving chapters parsing").c_str(), "Error",
						MB_OK | MB_ICONERROR);
					break;
				}
				++i;
			}

			// ----- Read [SPECIALS] section -----
			MovieDataForm->edSpecialsCDNumber->Text = pIniFile->ReadString("SPECIALS",
				"CD", "");
			i = 1;
			MovieDataForm->lstSpecials->Items->Clear();
			while( (strTmp = pIniFile->ReadString("SPECIALS", AnsiString(i), "")) != "" )
			{
				posSeparator = strTmp.Pos(" ");
				if( posSeparator > 0 )
				{
					// Add each special
					TListItem  *li = MovieDataForm->lstSpecials->Items->Add();
					li->Caption = AnsiString(i);
					li->SubItems->Add( strTmp.SubString(posSeparator+1,strTmp.Length() -
						posSeparator) );
					li->SubItems->Add( strTmp.SubString(1,posSeparator-1) );						
				}
				else
				{
					MessageBox(PageViewerForm->Handle, AnsiString("Malformed special entry : <" + strTmp +
						">, Leaving specials parsing").c_str(), "Error",
						MB_OK | MB_ICONERROR);
					break;
				}
				++i;
			}

			// ----- Read [LANGUAGES] section -----
			MovieDataForm->edLanguageDirectory->Text = pIniFile->ReadString("LANGUAGES", "Directory", "");
			MovieDataForm->chkMultipleAudioAVI->Checked = pIniFile->ReadBool("LANGUAGES", "MultipleAudioAVI", 0);
			// Read Primary
			strTmp = pIniFile->ReadString("LANGUAGES", "Primary", "");
			if(strTmp != "") {
				MovieDataForm->lstLanguages->Items->Clear();
				posSeparator = strTmp.Pos(" ");
				if( posSeparator > 0 )
				{
					TListItem  *li = MovieDataForm->lstLanguages->Items->Add();
					li->Caption = 0;
					li->SubItems->Add( strTmp.SubString(1,posSeparator-1) );
					li->SubItems->Add( strTmp.SubString(posSeparator+1,strTmp.Length() -
						posSeparator) );
					li->SubItems->Add( "" ); // no file for primary
					MovieDataForm->edLangFile1->Enabled = true;
					MovieDataForm->edLangFile1->Color = clWindow;
					MovieDataForm->bttBrowseLanguage1->Enabled = true;
				}
				else
				{
					MessageBox(PageViewerForm->Handle, AnsiString("Malformed primary language entry : <" +
						strTmp + ">").c_str(), "Error", MB_OK | MB_ICONERROR);
				}
			}
			// Read others
			i = 1;
			while( (strTmp = pIniFile->ReadString("LANGUAGES", AnsiString(i), "")) != "" )
			{
				posSeparator = strTmp.Pos(" ");
				if( posSeparator > 0 )
				{
					// Add each language
					TListItem  *li = MovieDataForm->lstLanguages->Items->Add();
					li->Caption = AnsiString(i);
					li->SubItems->Add( strTmp.SubString(1,posSeparator-1) );
					li->SubItems->Add( strTmp.SubString(posSeparator+1,strTmp.Length() -
						posSeparator) );
					li->SubItems->Add( "" );  // we write the file later
				}
				else
				{
					MessageBox(PageViewerForm->Handle, AnsiString("Malformed language entry : <" + strTmp +
						">, Leaving languages parsing").c_str(), "Error",
						MB_OK | MB_ICONERROR);
					break;
				}
				++i;
			}

			// ----- Read [MENU] section -----
			MovieDataForm->edMenuDirectory->Text = pIniFile->ReadString("MENU", "Directory", "");
			MovieDataForm->edMenuAVIName->Text = pIniFile->ReadString("MENU", "AVIName", "");
			MovieDataForm->edMenuSize->Text = pIniFile->ReadString("MENU", "Size", "");
			MovieDataForm->edMenuAudioName->Text = pIniFile->ReadString("MENU", "AudioName", "");
			MovieDataForm->edMenuStartPage->Text = pIniFile->ReadString("MENU", "StartPage", "");
			MovieDataForm->edMenuMenuPage->Text = pIniFile->ReadString("MENU", "MenuPage", "");
			MovieDataForm->edMenuChaptersPage->Text = pIniFile->ReadString("MENU", "ChaptersPage", "");
			MovieDataForm->edMenuSpecialsPage->Text = pIniFile->ReadString("MENU", "SpecialsPage", "");

			// ----- Read [SUBTITLES] section -----
			MovieDataForm->edSubtitlesDirectory->Text = pIniFile->ReadString("SUBTITLES", "Directory", "");
			MovieDataForm->edSubtitlesDialogString->Text = pIniFile->ReadString("SUBTITLES", "DialogString", "");
			MovieDataForm->edSubtitlesLines->Text = pIniFile->ReadString("SUBTITLES", "Lines", "");
			MovieDataForm->edSubtitlesMaxDisplayDuration->Text = pIniFile->ReadString("SUBTITLES", "MaxDisplayDuration", "");
			MovieDataForm->chkSubtitlesEstimateDispDuration->Checked = pIniFile->ReadBool("SUBTITLES", "EstimateDisplayDuration", 0);

			// Read 1, 2, 3
			i = 1;
			while( (strTmp = pIniFile->ReadString("SUBTITLES", AnsiString(i), "")) != "" )
			{
				posSeparator = strTmp.Pos(" ");
				if( posSeparator > 0 )
				{
					// Add each subtitles
					TListItem  *li = MovieDataForm->lstSubtitles->Items->Add();
					li->Caption = AnsiString(i);
					li->SubItems->Add( strTmp.SubString(1,posSeparator-1) );
					li->SubItems->Add( strTmp.SubString(posSeparator+1,strTmp.Length() -
						posSeparator) );
					// Make place for others options (we will get it later)
					for(int i = 0; i < 7; i++)
						li->SubItems->Add( "" );
				}
				else
				{
					MessageBox(PageViewerForm->Handle, AnsiString("Malformed subtitles entry : <" + strTmp +
						">, Leaving subtitles parsing").c_str(), "Error",
						MB_OK | MB_ICONERROR);
					break;
				}
				++i;
			}
		}
		__finally
		{
			delete pIniFile;
		}

		MDVDPIniFileReader* ifr;
		try
		{
			// Use our own ini file reader to read Page and Selection (we must keep comment) & File list (lang & sub)
			ifr = new MDVDPIniFileReader(OpenDialog1->FileName);
			TStringList* l = new TStringList();
			ifr->ParseFileList("[LANGUAGES]",l);
			for(int j = 0; j < l->Count; j++)
			{
				MovieDataForm->lstLanguages->Items->Item[j+1]->SubItems->Strings[2] = l->Strings[j];
			}
			l->Clear();

			ifr->ParseFileList("[SUBTITLES]",l);
			for(int j = 0; j < l->Count; j++)
			{
				TListItem* li = MovieDataForm->lstSubtitles->Items->Item[j];
				li->SubItems->Strings[2] = l->Strings[j];
				if(l->Objects[j] != NULL)
				{
					TStringList* lsub = (TStringList*)l->Objects[j];
					li->SubItems->Strings[3] = lsub->Values["Format"];
					li->SubItems->Strings[4] = lsub->Values["OpenBracket"];
					li->SubItems->Strings[5] = lsub->Values["CloseBracket"];
					li->SubItems->Strings[6] = lsub->Values["LineSeparator"];
					li->SubItems->Strings[7] = lsub->Values["Offset"];
					li->SubItems->Strings[8] = lsub->Values["Multiplier"];
					delete lsub;
				}
			}
			delete l;

			ifr->OnNewSelectionRead = OnNewSelectionReadInINIFile;
			ifr->OnNewPageRead = OnNewPageReadInINIFile;

			ShowCrossCursor(false);
			if( !MovieDataForm->edMenuAVIName->Text.IsEmpty() )
			{
				OpenAVI(MovieDataForm->edMenuDirectory->Text + "\\" +
					MovieDataForm->edMenuAVIName->Text);
				MainForm->Refresh();
				PageViewerForm->Refresh();
			} else {
				// Look if an AVI is already open
				if(pAvifile != NULL)
				{
					// Free old file ressources
					AVIStreamGetFrameClose(pGetFrame);
					AVIStreamRelease(pAviStream);
					AVIFileRelease(pAvifile);  // closes the file
					AVIFileExit();          // releases AVIFile library
					pAvifile = NULL;
					pAviStream = NULL;
					pGetFrame = NULL;
				}
			}
			ifr->ParsePagesAndSelections();
			OutputDebugString("ParsePagesAndSelections terminated");
			PageViewerForm->cbJumpAtEnd->Items->Assign(PageViewerForm->PageList->Items);
			PageViewerForm->PageList->ItemIndex = 0;
			PageViewerForm->ShowPage(PageViewerForm->PageList->ItemIndex);
			VideoTrackBar->OnChange(this);
			ShowCrossCursor(true);
		}
		catch(MDVDPFileException& e)
		{
			MessageBox( PageViewerForm->Handle, e.msg.c_str(), "Error", MB_OK | MB_ICONERROR );
		}
	delete ifr;
	}
}

//---------------------------------------------------------------------------

AnsiString __fastcall TMainForm::GetCurrentShapeXYXYCoordinate(void)
{
	// Calculate coordinate of shape
	int left = currentShape->Left - VideoImage->Left;
	int top = currentShape->Top - VideoImage->Top;
	int right = currentShape->Width + left;
	int bottom = currentShape->Height + top;

	return AnsiString(left) + "," + top + "-" + right + "," + bottom;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miMMSaveasINIClick (TObject * Sender)
{
	SaveINI(true);
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miMovieDataClick(TObject *Sender)
{
	AnsiString OriginalMenuName = MovieDataForm->edMenuAVIName->Text;
	MovieDataForm->ShowModal();
	// check if the menu avi has changed and load it in case	
	if(MovieDataForm->edMenuAVIName->Text != OriginalMenuName)
	{
			OpenAVI( MovieDataForm->edBaseDirectory->Text +
				MovieDataForm->edMenuDirectory->Text + "\\" +
				MovieDataForm->edMenuAVIName->Text );
			miPage->Enabled = true;
	}

}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miNewClick(TObject *Sender)
{
	// Clear All
	PageViewerForm->ReInit();
	SelectionsViewerForm->ReInit();
	// To much things to reinit here so delete & recreate form
	delete MovieDataForm;
	MovieDataForm = new TMovieDataForm(this);

	CurrentFileName = "New File";
	UpdateCaption();

	VideoImage->Canvas->Brush->Color = clBtnFace;
	VideoImage->Canvas->FillRect( VideoImage->Canvas->ClipRect );
	// Look if an AVI is already open
	if(pAvifile != NULL)
	{
		// Free old file ressources
		AVIStreamGetFrameClose(pGetFrame);
		AVIStreamRelease(pAviStream);
		AVIFileRelease(pAvifile);  // closes the file
		AVIFileExit();          // releases AVIFile library
		pAvifile = NULL;
		pAviStream = NULL;
		pGetFrame = NULL;
	}

	// Start a new INI, we must have a base directory
	if( MovieDataForm->ShowModal() == mrOk)
	{
		// Open menu video if it exist
		if( !MovieDataForm->edMenuAVIName->Text.IsEmpty() )
		{
			OpenAVI( MovieDataForm->edBaseDirectory->Text +
				MovieDataForm->edMenuDirectory->Text + "\\" +
				MovieDataForm->edMenuAVIName->Text );
			miPage->Enabled = true;
		}
		miMovieData->Enabled = true;
		miMMSaveINI->Enabled = true;		
		miMMSaveasINI->Enabled = true;
	}
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::ShowMessageMissINIKey(AnsiString as)
{
	MessageBox(PageViewerForm->Handle, as.c_str(), "Error", MB_OK | MB_ICONERROR);
	return;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miMMSaveINIClick(TObject *Sender)
{
	SaveINI( CurrentFileName == "New File" );
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveINI(bool ForceSaveAs)
{
	if( ForceSaveAs )
	{
		SaveDialog1->Filter = "MDVD menu INI file (*.ini)|*.INI";
		SaveDialog1->DefaultExt = "INI";
		if( SaveDialog1->Execute() )
		{
			CurrentFileName = SaveDialog1->FileName;
		} else {
			// User as canceled
			return;
		}
	}

	TStringList *fINI = new TStringList ();

	fINI->Add ("[Micro DVD Ini File]");
	fINI->Add ("");

	// --------------- [MAIN] ---------------
  fINI->Add ("[MAIN]");
  fINI->Add ("Title=" + MovieDataForm->edMainTitle->Text);
	fINI->Add ("ID=" + MovieDataForm->edMainID->Text);
	AddIfParamNotEmpty(fINI, "CDNumber", MovieDataForm->edMainCDNumber->Text);
	AddIfParamNotEmpty(fINI, "Delay", MovieDataForm->edMainDelay->Text);
	fINI->Add ("");

	// --------------- [MOVIE] ---------------
	fINI->Add ("[MOVIE]");
	fINI->Add ("Directory=" + MovieDataForm->edMovieDirectory->Text);
	if (MovieDataForm->edMovieAVIName->Text.IsEmpty ())
	{
		ShowMessage("You must have at least 1 video file in your movie.");
		delete fINI;
		return;
	}
	fINI->Add ("AVIName=" + MovieDataForm->edMovieAVIName->Text);
	AddIfParamNotEmpty(fINI, "AVI2Name", MovieDataForm->edMovieAVI2Name->Text);
	AddIfParamNotEmpty(fINI, "AVI3Name", MovieDataForm->edMovieAVI3Name->Text);
	// TODO : we could check if this is multiple CD, so we must
	// have at least CD1Frames & CD2Frames
	AddIfParamNotEmpty(fINI, "CD1Frames", MovieDataForm->edMovieCD1Frames->Text);
	AddIfParamNotEmpty(fINI, "CD2Frames", MovieDataForm->edMovieCD2Frames->Text);
	AddIfParamNotEmpty(fINI, "CD3Frames", MovieDataForm->edMovieCD3Frames->Text);
	fINI->Add ("");

  // --------------- [CHAPTERS] ---------------
  if (MovieDataForm->lstChapters->Items->Count > 0)
  {
    fINI->Add ("[CHAPTERS]");
    for (int i = 0; i < MovieDataForm->lstChapters->Items->Count; i++)
    {
      TListItem *li = MovieDataForm->lstChapters->Items->Item[i];

			fINI->Add (li->Caption + "=" + li->SubItems->Strings[0] + " " +
		 		li->SubItems->Strings[1]);
    }
		fINI->Add ("");
	}

  // --------------- [SPECIALS] ---------------
  if (MovieDataForm->lstSpecials->Items->Count > 0)
  {
    fINI->Add ("[SPECIALS]");
    fINI->Add ("CD=" + MovieDataForm->edSpecialsCDNumber->Text);
    for (int i = 0; i < MovieDataForm->lstSpecials->Items->Count; i++)
    {
			TListItem *li = MovieDataForm->lstSpecials->Items->Item[i];
			fINI->Add (li->Caption + "=" + li->SubItems->Strings[1] + " " +
		 		li->SubItems->Strings[0]);
    }
    fINI->Add ("");
	}

  // --------------- [LANGUAGES] ---------------
	if (MovieDataForm->lstLanguages->Items->Count > 0)
  {
    fINI->Add ("[LANGUAGES]");
		fINI->Add ("Directory=" + MovieDataForm->edLanguageDirectory->Text);
		if(MovieDataForm->chkMultipleAudioAVI->Checked)
			fINI->Add ("MultipleAudioAVI=1");
    for (int i = 0; i < MovieDataForm->lstLanguages->Items->Count; i++)
    {
			TListItem *li = MovieDataForm->lstLanguages->Items->Item[i];
      if (i == 0)
      {
				fINI->Add ("Primary=" + li->SubItems->Strings[0] + " " +
					li->SubItems->Strings[1]);
			}
			else
			{
				fINI->Add (li->Caption + "=" + li->SubItems->Strings[0] + " " +
					li->SubItems->Strings[1]);
				fINI->Add ("File=" + li->SubItems->Strings[2]);
			}
		}
		fINI->Add ("");
	}

	// --------------- [SUBTITLES] ---------------
	if (MovieDataForm->lstSubtitles->Items->Count > 0)
  {
		fINI->Add ("[SUBTITLES]");
		fINI->Add ("Directory=" + MovieDataForm->edSubtitlesDirectory->Text);
		AddIfParamNotEmpty(fINI, "DialogString", MovieDataForm->edSubtitlesDialogString->Text);
		AddIfParamNotEmpty(fINI, "Lines", MovieDataForm->edSubtitlesLines->Text);
		AddIfParamNotEmpty(fINI, "MaxDisplayDuration", MovieDataForm->edSubtitlesMaxDisplayDuration->Text);
		if (MovieDataForm->chkSubtitlesEstimateDispDuration->Checked)
			fINI->Add ("EstimateDisplayDuration=1");
    for (int i = 0; i < MovieDataForm->lstSubtitles->Items->Count; i++)
    {
      TListItem *li = MovieDataForm->lstSubtitles->Items->Item[i];

      fINI->Add (li->Caption + "=" + li->SubItems->Strings[0] + " " +
				li->SubItems->Strings[1]);
			fINI->Add ("File=" + li->SubItems->Strings[2]);
			AddIfParamNotEmpty(fINI, "Format", li->SubItems->Strings[3]);
			AddIfParamNotEmpty(fINI, "OpenBracket", li->SubItems->Strings[4]);
			AddIfParamNotEmpty(fINI, "CloseBracket", li->SubItems->Strings[5]);
			AddIfParamNotEmpty(fINI, "LineSeparator", li->SubItems->Strings[6]);
			AddIfParamNotEmpty(fINI, "Offset", li->SubItems->Strings[7]);
			AddIfParamNotEmpty(fINI, "Multiplier", li->SubItems->Strings[8]);
		}
    fINI->Add ("");
  }

	// --------------- [MENU] ---------------
	if( !MovieDataForm->edMenuDirectory->Text.IsEmpty() )
	{
		fINI->Add ("[MENU]");
		fINI->Add ("Directory=" + MovieDataForm->edMenuDirectory->Text);
		AddIfParamNotEmpty(fINI, "AVIName", MovieDataForm->edMenuAVIName->Text);
		AddIfParamNotEmpty(fINI, "AudioName", MovieDataForm->edMenuAudioName->Text);
		AddIfParamNotEmpty(fINI, "Size", MovieDataForm->edMenuSize->Text);
		if (MovieDataForm->edMenuStartPage->Text.IsEmpty ())
		{
			ShowMessage("You must set the menu 'start page'.");
			delete fINI;
			return;
		}
		fINI->Add ("StartPage=" + MovieDataForm->edMenuStartPage->Text);
		if (MovieDataForm->edMenuMenuPage->Text.IsEmpty ())
		{
			ShowMessage("You must set the menu 'menu page'.");
			delete fINI;
			return;
		}
		fINI->Add ("MenuPage=" + MovieDataForm->edMenuMenuPage->Text);
		AddIfParamNotEmpty(fINI, "ChaptersPage", MovieDataForm->edMenuChaptersPage->Text);
		AddIfParamNotEmpty(fINI, "SpecialsPage", MovieDataForm->edMenuSpecialsPage->Text);
		fINI->Add ("");
	}

  AnsiString tmp;
  Page   *p;

	for (int i = 0; i < PageViewerForm->PageList->Items->Count; i++)
  {
    p = (Page *) PageViewerForm->PageList->Items->Objects[i];

    tmp.sprintf ("[PAGE=%d]\t\t;%s", p->Number, p->Name);
    fINI->Add (tmp);

		fINI->Add ("\tBackgroundMode=" + AnsiString (p->isVideoType ? "1" : "2"));

    if (!p->isVideoType)
    {
			fINI->Add ("\tPictureFile=" + p->PictureFileName);
			fINI->Add ("\tPictureDuration=" + AnsiString(p->PictureDuration) );
		}
		else
		{
      fINI->Add ("\tVideoLoop=" + AnsiString (p->VideoLoop ? "1" : "0"));

			tmp.sprintf ("%d-%d", p->FirstFrameNumber, p->LastFrameNumber);
			fINI->Add ("\tVideoRange=" + tmp);

			if (p->ActionAtEnd != NULL)
				fINI->Add ("\tActionAtEnd=" + AnsiString(p->ActionAtEnd->GetRealType()) +
				AnsiString("#") + p->ActionAtEnd->param);

			if (p->JumpAtEnd != 0) {
				fINI->Add ("\tJumpAtEnd=" + AnsiString(p->JumpAtEnd) );
			}

		}

		fINI->Add ("\tSelectionAvailable=" +
			AnsiString (p->SelectionAvailable ? "1" : "0"));

		if (p->SelectionAvailable)
		{
			if (p->DefaultSelection > 0 &&
          (unsigned int)p->DefaultSelection <= p->vectSel.size() )
			{
				fINI->Add ("\tDefaultSelection=" +
					AnsiString(p->DefaultSelection) );
			} else {
				fINI->Add ("\tDefaultSelection=-1");
			}

			if (p->ProhibitSelectionBefore != 0)
				fINI->Add ("\tProhibitSelectionBefore=" +
					AnsiString(p->ProhibitSelectionBefore) );

			if (p->ProhibitSelectionAfter != 0)
				fINI->Add ("\tProhibitSelectionAfter=" +
					AnsiString(p->ProhibitSelectionAfter) );

			if(p->ColumnOrdering)
				fINI->Add ("\tSelectionOrdering=2");
		}

    for (unsigned int j = 0; j < p->vectSel.size (); j++)
    {
			tmp.sprintf ("\t[SELECTION=%d]\t\t;%s", j + 1, p->vectSel[j]->Name);
      fINI->Add (tmp);
      fINI->Add ("\t\tArea=" + p->vectSel[j]->Area);
      fINI->Add ("\t\tAction=" + p->vectSel[j]->GetActionTypeString ());
      fINI->Add ("\t\tParameter=" + p->vectSel[j]->GetActionParamString ());
    }

    fINI->Add ("");
  }

	fINI->SaveToFile (CurrentFileName);

	delete fINI;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::UpdateCaption()
{
	Caption = Application->Title + " - " + CurrentFileName;
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::miAboutClick(TObject *Sender)
{
	AboutForm->ShowModal();
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveOptions(void)
{
	TRegistry *Reg = new TRegistry();
	try
	{
		Reg->RootKey = HKEY_CURRENT_USER;

		if (Reg->OpenKey("\\Software\\MDVDPMenuMaker", true))
		{
			Reg->WriteInteger("Top",Top);
			Reg->WriteInteger("Left",Left);
			if(MovieDataForm->edBaseDirectory->Text.IsEmpty());
				Reg->WriteString("LastDir",MovieDataForm->edBaseDirectory->Text);
			Reg->CloseKey();
		}
	}
	__finally {	delete Reg; }
}

//---------------------------------------------------------------------------

void __fastcall TMainForm::LoadOptions(void)
{
	TRegistry *Reg = new TRegistry();
	try
	{
		Reg->RootKey = HKEY_CURRENT_USER;

		if (Reg->OpenKey("\\Software\\MDVDPMenuMaker", false))
		{
			try	{
				Top = Reg->ReadInteger("Top");
			} catch(ERegistryException &e) {
				Top = 100;
			}

			try	{
				Left = Reg->ReadInteger("Left");
			} catch(ERegistryException &e) {
				Left = 100;
			}

			try {
				SetCurrentDir(Reg->ReadString("LastDir"));
			} catch(ERegistryException &e) { }

			Reg->CloseKey();
		}

	}
	__finally {	delete Reg; }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddIfParamNotEmpty(TStringList* fINI, AnsiString key,
	AnsiString param)
{
	if( !param.IsEmpty() )
	{
		fINI->Add(key + "=" + param);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnChangePageSelectionClick(TObject* Sender)
{
	TMenuItem* mi = (TMenuItem*)Sender;

	Selection* s = (Selection*)currentShape->Tag;

	Page* p = (Page*)PageViewerForm->PageList->Items->Objects[
		PageViewerForm->PageList->ItemIndex ];
	// Search the selection in vector
	vector<Selection*>::iterator si = find(p->vectSel.begin(), p->vectSel.end(), s);
	// Remove selection of vector
	p->vectSel.erase( si );

	// Add selection to page destination vector
	Page* dstP = (Page*) mi->Tag;
 	dstP->vectSel.push_back(s);
	dstP->SelectionNumber++;

	// Delete current shape
	delete currentShape;
	currentShape = NULL;

	// Refresh page selection list
	SelectionsViewerForm->ShowPageSelections(p);
	PageViewerForm->UpdateDefaultSelectionCombo(p);
}
//---------------------------------------------------------------------------
/*
int __fastcall TMainForm::CalculateChapterMax()
{
	Page* p;
	Selection *s;
	MenuAction *ma;

	int ChapterMax = 0;

	for(int i = 0; i < PageViewerForm->PageList->Items->Count; i++)
	{
		p = (Page*)PageViewerForm->PageList->Items->Objects[i];
		for(unsigned int j = 0; j < p->vectSel.size(); j++)
		{
			s = p->vectSel[j];
			for(unsigned int k = 0; k < s->vectAction.size(); k++)
			{
				ma = s->vectAction[k];
				if(ma->type == PLAY_CHAPTER)
				{
					int ChapterNum = ma->param.ToInt();
					if(ChapterNum > ChapterMax)
						ChapterMax = ChapterNum;
				}
			}
		}
	}
}
*/
//---------------------------------------------------------------------------
void __fastcall TMainForm::DockPageControlDockDrop(TObject *Sender,
      TDragDockObject *Source, int X, int Y)
{
	// on first dock
	if(DockPageControl->DockClientCount == 1)
	{
		DockPageControl->Width = Source->Control->UndockWidth;
		Splitter1->Left = Source->Control->UndockWidth;
		Splitter1->Visible = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DockPageControlUnDock(TObject *Sender,
			TControl *Client, TWinControl *NewTarget, bool &Allow)
{
	// on last undock
	if(DockPageControl->DockClientCount == 1)
	{
		DockPageControl->Width = 10;
		Splitter1->Visible = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::miDockedClick(TObject *Sender)
{
	if(miDocked->Checked)
	{
		miDocked->Checked = false;
		Splitter1->Visible = false;		
		DockPageControl->Visible = false;
		MainForm->Width -= DockPageControl->Width;
		DockPageControl->Width = 0;
		PageViewerForm->ManualDock(NULL,NULL,NULL);
		SelectionsViewerForm->ManualDock(NULL,NULL,NULL);
		// Put subform around MainForm
		OnMainFormMove(TMessage());
	} else {
		miDocked->Checked = true;
		int dockWidth = max(PageViewerForm->Width,
			SelectionsViewerForm->Width);
		MainForm->Width += dockWidth;
		DockPageControl->Width = dockWidth;
		DockPageControl->Visible = true;
		Splitter1->Left = DockPageControl->Width + 1;
		Splitter1->Visible = true;
		PageViewerForm->ManualDock(DockPageControl,NULL,NULL);
		SelectionsViewerForm->ManualDock(DockPageControl,NULL,NULL);
		DockPageControl->ActivePageIndex = 0;		
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miGotoNextPageClick(TObject *Sender)
{
	if(PageViewerForm->PageList->ItemIndex < PageViewerForm->PageList->Items->Count)
		PageViewerForm->PageList->ItemIndex++;
	else
		PageViewerForm->PageList->ItemIndex = 0;
	PageViewerForm->ShowPage( PageViewerForm->PageList->ItemIndex );
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miGotoPreviousPageClick(TObject *Sender)
{
	if(PageViewerForm->PageList->ItemIndex > 0)
		PageViewerForm->PageList->ItemIndex--;
	else
		PageViewerForm->PageList->ItemIndex = 0;
	PageViewerForm->ShowPage( PageViewerForm->PageList->ItemIndex );
}
//---------------------------------------------------------------------------


