/***************************************************************************\
 * Micro DVD Menu Maker by Christophe.Paris@free.fr                        *
 ***************************************************************************
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
\***************************************************************************/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "Page.h"
#include "PageViewer.h"
#include "SelectionViewer.h"
#include "MovieData.h"

#include <stdio.h> // for sscanf
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPageViewerForm *PageViewerForm;

//---------------------------------------------------------------------------

__fastcall TPageViewerForm::TPageViewerForm(TComponent* Owner)
	: TForm(Owner)
{
	Top  = MainForm->Top;
	Left = MainForm->Left + MainForm->Width;

	cbJumpAtEnd->ItemIndex = 0;
	cbDefaultSelection->ItemIndex = 0;

	ClipboardPageIndex = -1;
	previousPageIndex = -1;
	currentPageIndex = -1;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::PageListDrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
	PageList->Canvas->FillRect(Rect); // clear the rectangle

	Page* p=(Page*)PageList->Items->Objects[Index];

	// Show Image
	int topBorderImg = (Rect.Height() - p->GetSmallImage()->Graphic->Height) / 2;
	PageList->Canvas->Draw(Rect.Left + 2, Rect.Top + topBorderImg,
		p->GetSmallImage()->Graphic);

	// Show copy icon
	if(Index == ClipboardPageIndex)
		ImageList1->Draw(PageList->Canvas, Rect.Left + 72, Rect.Top + 4, 0, true);

	// Show start icon
	if( MovieDataForm->edMenuStartPage->Text == p->Number )
		ImageList1->Draw(PageList->Canvas, Rect.Left + 96, Rect.Top + 4, 5, true);

	// Show menu icon
	if( MovieDataForm->edMenuMenuPage->Text == p->Number )
		ImageList1->Draw(PageList->Canvas, Rect.Left + 110, Rect.Top + 4, 6, true);

	// Show chapters icon
	if( MovieDataForm->edMenuChaptersPage->Text == p->Number )
		ImageList1->Draw(PageList->Canvas, Rect.Left + 124, Rect.Top + 4, 7, true);

	// Show specials icon
	if( MovieDataForm->edMenuSpecialsPage->Text == p->Number )
		ImageList1->Draw(PageList->Canvas, Rect.Left + 138, Rect.Top + 4, 8, true);

	// Show bottom black thin line
	PageList->Canvas->MoveTo(Rect.Left,Rect.Bottom - 1);
	PageList->Canvas->LineTo(Rect.Right,Rect.Bottom - 1);

	// Show the Page's name
	int topBorderTxt = (Rect.Height() + PageList->Canvas->Font->Height) / 2;

	PageList->Canvas->TextOut(Rect.Left + 72, Rect.Top + topBorderTxt, (*p));
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::bttAddClick(TObject *Sender)
{
	AnsiString pageName;

	int ff = 0, lf = 0;
	sscanf(edRange->Text.c_str(),"%d-%d",&ff,&lf);

	// Goto first in range
	if(rbVideo->Checked && ff != 0)
	{
		MainForm->VideoTrackBar->Position = ff;
	}

	// Create a temporary small image
	TImage *smallImage = new TImage(PageList);
	// Specify new dimmension
	TRect DestRect;
	DestRect.Left = 0; DestRect.Top = 0;
	DestRect.Right = 64; DestRect.Bottom = 43;
	// Copy the image and resize it
	smallImage->Canvas->StretchDraw(DestRect,
		MainForm->VideoImage->Picture->Graphic);
	smallImage->Picture->Bitmap->Width = 64;
	smallImage->Picture->Bitmap->Height = 43;

	// Create the new page
	if(edPageName->Text.IsEmpty())
	{
		// No name given so give one
		pageName = "Untitled";
	}
	else
	{
		pageName = edPageName->Text;
	}

	// Calculate the page number
	int pageNumber = 0;
	if( ! edNumber->Text.IsEmpty() )
	{
		// A page number is provided
		try
		{
			pageNumber = edNumber->Text.ToInt();
			// check if the page number doesn't exist already
			for(int i=0; i < PageList->Items->Count; i++)
			{
				if( ((Page*)PageList->Items->Objects[i])->Number == pageNumber )
				{
					// The page number already exist
					pageNumber = 0;
					break;
				}
			}
		}
		catch(EConvertError& e)
		{
		}
	}
	if(pageNumber == 0) {
		// 0 mean that no page number is given yet
		// Take the max page number + 1
		int max = 0, tmpNum;

		for(int i=0; i < PageList->Items->Count; i++)
		{
			tmpNum = ((Page*)PageList->Items->Objects[i])->Number;
			if( tmpNum > max )
				max = tmpNum;
		}
		pageNumber = max + 1;
	}

	Page* p = new Page(pageNumber, pageName, smallImage->Picture);
	p->isVideoType = rbVideo->Checked;
	p->SelectionAvailable = chkSelectionAvailable->Checked;

	// Create new page
	if(rbVideo->Checked)
	{
		p->FirstFrameNumber = ff;
		p->LastFrameNumber = lf;
		p->VideoLoop = chkVideoLoop->Checked;
	}
	else
	{
		p->PictureFileName = edPictureFileName->Text;
	}

	try
	{
		if( !edProhibitSelBefore->Text.IsEmpty() )
			p->ProhibitSelectionBefore = edProhibitSelBefore->Text.ToInt();
	} catch(...) { }

	try
	{
		if( !edProhibitSelAfter->Text.IsEmpty() )
			p->ProhibitSelectionAfter = edProhibitSelAfter->Text.ToInt();
	} catch(...) { }

	p->ColumnOrdering = rbColumn->Checked;

	PageList->Items->AddObject(p->Number, (TObject*)p);

	// Free temporary small image
	delete smallImage;

	PageList->ItemIndex = PageList->Items->Count - 1;

	// Update JumpAtEnd combo box
	cbJumpAtEnd->Items->Assign(PageList->Items);

	ShowPage( PageList->ItemIndex );
}
//---------------------------------------------------------------------------
void __fastcall TPageViewerForm::bttDelClick(TObject *Sender)
{
	DeletePage( PageList->ItemIndex );
	ClipboardPageIndex = -1;
	ShowPage( PageList->ItemIndex );
}
//---------------------------------------------------------------------------
void __fastcall TPageViewerForm::bttFirstClick(TObject *Sender)
{
	int ff,lf;

	if( sscanf(edRange->Text.c_str(),"%d-%d",&ff,&lf) )
	{
		edRange->Text = MainForm->edVideoPos->Text + "-" + lf;
	}
/*
	if(PageList->ItemIndex == -1)
    	return;

	int newFirstFrame = MainForm->edVideoPos->Text.ToInt();
    Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];

	if(newFirstFrame < p->LastFrameNumber)
    {
    	p->FirstFrameNumber = newFirstFrame;
        PageList->Refresh();
    } else {
    	MessageBox(Handle,"The first page frame must be before last frame.",
        	"Error",MB_OK|MB_ICONERROR);
    }
*/
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::bttLastClick(TObject *Sender)
{
	int ff,lf;

	if( sscanf(edRange->Text.c_str(),"%d-%d",&ff,&lf) )
	{
		edRange->Text = AnsiString(ff) + "-" + MainForm->edVideoPos->Text;
	}

/*	if(PageList->ItemIndex == -1)
    	return;

	int newLastFrame = MainForm->edVideoPos->Text.ToInt();
    Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];

	if(newLastFrame > p->FirstFrameNumber)
    {
    	p->LastFrameNumber = newLastFrame;
        PageList->Refresh();
    } else {
    	MessageBox(Handle,"The last page frame must be after first frame.",
        	"Error",MB_OK|MB_ICONERROR);
    }
*/
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::PageListClick(TObject *Sender)
{
	ShowPage( PageList->ItemIndex );
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::bttModifyClick(TObject *Sender)
{
	if(PageList->ItemIndex == -1)
		return;

	Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];

	// Update name
	if(!edPageName->Text.IsEmpty())
	{
		// TODO : Also check name
		// TODO : Change page name in all others pages & selections
		p->Name = edPageName->Text;
	}
	// Update Video Type
	p->isVideoType = rbVideo->Checked;
	// Update picture file
	if(edPictureFileName->Text != p->PictureFileName) {

		// Remake small Image
		TImage *smallImage = new TImage(PageList);
		// Specify new dimmension
		TRect DestRect;
		DestRect.Left = 0; DestRect.Top = 0;
		DestRect.Right = 64; DestRect.Bottom = 43;
		// Copy the image and resize it
		smallImage->Canvas->StretchDraw(DestRect,
			MainForm->VideoImage->Picture->Graphic);
		smallImage->Picture->Bitmap->Width = 64;
		smallImage->Picture->Bitmap->Height = 43;

		p->SetSmallImage(smallImage->Picture);
		p->PictureFileName = edPictureFileName->Text;

		delete smallImage;
	}
	// Update Selection available flag
	p->SelectionAvailable = chkSelectionAvailable->Checked;

	p->VideoLoop = chkVideoLoop->Checked;

	// Update Range
	int ff,lf;
	if( sscanf(edRange->Text.c_str(),"%d-%d",&ff,&lf) )
	{
		p->FirstFrameNumber = ff;
		p->LastFrameNumber = lf;
	}

	// Update Picture Duration
	try
	{
		if( !edPictureDuration->Text.IsEmpty() )
			p->PictureDuration = edPictureDuration->Text.ToInt();
	} catch(...) { }

	if(cbJumpAtEnd->ItemIndex != -1)
		p->JumpAtEnd = ((Page*)cbJumpAtEnd->Items->Objects[cbJumpAtEnd->ItemIndex])->Number;

	// Update Prohibit Selection *
	try
	{
		if( !edProhibitSelBefore->Text.IsEmpty() )
			p->ProhibitSelectionBefore = edProhibitSelBefore->Text.ToInt();
	} catch(...) { }
	
	try
	{
		if( !edProhibitSelAfter->Text.IsEmpty() )
			p->ProhibitSelectionAfter = edProhibitSelAfter->Text.ToInt();
	} catch(...) { }

	p->ColumnOrdering = rbColumn->Checked;	

	// Update JumpAtEnd combo box
	int savidx = cbJumpAtEnd->ItemIndex;
	cbJumpAtEnd->Items->Assign(PageList->Items);
	cbJumpAtEnd->ItemIndex = savidx;

	// Update DefaultSelection
	p->DefaultSelection = cbDefaultSelection->ItemIndex;

	PageList->Refresh();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::bttShowAdvancedOptClick(TObject *Sender)
{
	static int ExpandedStartSize = gbOptions->Height;
	static bool expanded = true;

	int offset = ExpandedStartSize - (bttShowAdvancedOpt->Top + bttShowAdvancedOpt->Height + 10);

	offset = (expanded) ? -offset : offset;

	gbOptions->Height += offset;
	plBottControl->Height += offset;
	PageViewerForm->Height += offset;

	bttShowAdvancedOpt->Caption = expanded ? ">>" : "<<";
	bttShowAdvancedOpt->Hint = expanded ? "Show advanced options" : "Hide advanced options";

	expanded = !expanded;

	if(MainForm->miDocked->Checked)
	{
		PageList->Realign();		
	}
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::rbVideoClick(TObject *Sender)
{
	lbPicture->Enabled = rbStillImage->Checked;
  edPictureFileName->Enabled = rbStillImage->Checked;
  bttBrowsePicture->Enabled = rbStillImage->Checked;
  lbPictureDuration->Enabled = rbStillImage->Checked;
  edPictureDuration->Enabled = rbStillImage->Checked;

  chkVideoLoop->Enabled = rbVideo->Checked;
  lbRange->Enabled = rbVideo->Checked;
  edRange->Enabled = rbVideo->Checked;
  bttFirst->Enabled = rbVideo->Checked;
  bttLast->Enabled = rbVideo->Checked;

  lbPSB->Enabled = chkSelectionAvailable->Checked && rbVideo->Checked;
  edProhibitSelBefore->Enabled = chkSelectionAvailable->Checked &&
		rbVideo->Checked;
	edProhibitSelAfter->Enabled = chkSelectionAvailable->Checked &&
		rbVideo->Checked;

	lbSelOrder->Enabled = chkSelectionAvailable->Checked;
	rbRow->Enabled = chkSelectionAvailable->Checked;
	rbColumn->Enabled = chkSelectionAvailable->Checked;

  lbDefaultSel->Enabled = chkSelectionAvailable->Checked;
  cbDefaultSelection->Enabled = chkSelectionAvailable->Checked;

	chkVideoLoop->Checked = rbVideo->Checked;
	
	lbJumpAtEnd->Enabled = !rbVideo->Checked;
	cbJumpAtEnd->Enabled = !rbVideo->Checked;
	lbActionAtEnd->Enabled = !rbVideo->Checked;
	lstActionAtEnd->Enabled = !rbVideo->Checked;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::bttBrowsePictureClick(TObject *Sender)
{
	MainForm->OpenDialog1->FileName = "";
	MainForm->OpenDialog1->Filter = "JPG Pictures files (*.jpg)|*.JPG";
	MainForm->OpenDialog1->InitialDir = MovieDataForm->edMenuDirectory->Text;
	if(MainForm->OpenDialog1->Execute())
	{
		AnsiString menuDir = MovieDataForm->edBaseDirectory->Text +
			MovieDataForm->edMenuDirectory->Text;
		menuDir = IncludeTrailingBackslash(menuDir);
		menuDir = StringReplace (menuDir, "\\.\\", "\\",
			TReplaceFlags () << rfReplaceAll);

		// Check if the file is son of the menu directory
		if( MainForm->OpenDialog1->FileName.SubString(1, menuDir.Length() ) != menuDir )
		{
			MessageBox(Handle, AnsiString("The picture file must be in the menu directory.").c_str(), "Error", MB_OK|MB_ICONERROR);
			return;
		}

		edPictureFileName->Text = ExtractFileName( MainForm->OpenDialog1->FileName );
		edPageName->Text = ChangeFileExt(edPictureFileName->Text,"");

		TJPEGImage* JpegImg = new TJPEGImage();
		JpegImg->LoadFromFile(MainForm->OpenDialog1->FileName);
		// TODO : check image size
		JpegImg->DIBNeeded();
		MainForm->VideoImage->Picture->Bitmap->Assign(JpegImg);
		delete JpegImg;

		MovieDataForm->edMenuSize->Text = AnsiString().sprintf("%d,%d",
			MainForm->VideoImage->Picture->Width,
			MainForm->VideoImage->Picture->Height);
			
		MainForm->ResizeVideoImage(MainForm->VideoImage->Picture->Width,
			MainForm->VideoImage->Picture->Height);
	}
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::FormDestroy(TObject *Sender)
{
	for(int i=0; i < PageList->Items->Count; i++)
	{
		delete (Page*)PageList->Items->Objects[i];
	}

	PageList->Clear();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::chkVideoLoopClick(TObject *Sender)
{
	lbJumpAtEnd->Enabled = ! chkVideoLoop->Checked;
	cbJumpAtEnd->Enabled = lbJumpAtEnd->Enabled;

	lbActionAtEnd->Enabled = !chkVideoLoop->Checked;
	lstActionAtEnd->Enabled = lbActionAtEnd->Enabled;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::puPLDeleteClick(TObject *Sender)
{
	DeletePage( PageList->ItemIndex );
	ShowPage( PageList->ItemIndex );
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::puPLUpDownClick(TObject *Sender)
{
	TMenuItem *mi = (TMenuItem*)Sender;
	int UpDownOffset = mi->Tag;
	int newIndex = currentPageIndex + UpDownOffset;

	//Check if the newIndex is bounded
	if( newIndex < 0 || newIndex > (PageList->Items->Count - 1) )
	{
		return;
	}

	PageList->Items->Exchange(currentPageIndex, newIndex);

	// Update JumpAtEnd combo box
	cbJumpAtEnd->Items->Assign(PageList->Items);

	currentPageIndex = newIndex;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::cbJumpAtEndDrawItem(TWinControl *Control,
	int Index, TRect &Rect, TOwnerDrawState State)
{
	cbJumpAtEnd->Canvas->FillRect(Rect); // clear the rectangle
	Page* p=(Page*)cbJumpAtEnd->Items->Objects[Index];
	if( p != NULL)
	{
		cbJumpAtEnd->Canvas->TextOut(Rect.Left + 2, Rect.Top + 1,
			p->GetShortName());
	}
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::lstActionAtEndDragOver(TObject *Sender,
	TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
	// Check drag source, and accept
	Accept = ( Source == dynamic_cast<TObject *>(SelectionsViewerForm->DragDropActionPanel) ) &&
		(PageList->ItemIndex != -1);
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::lstActionAtEndDragDrop(TObject *Sender,
	TObject *Source, int X, int Y)
{
	// TODO : check parameters
	if(PageList->ItemIndex == -1)
		return;

	Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];

	MenuAction* a = SelectionsViewerForm->GetDragedAction();

	lstActionAtEnd->Clear();
	lstActionAtEnd->Items->AddObject("",(TObject*)a);
	lstActionAtEnd->Refresh();

	if(p->ActionAtEnd != NULL)
		delete p->ActionAtEnd;
	p->ActionAtEnd = a;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::lstActionAtEndDrawItem(
	TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
	lstActionAtEnd->Canvas->FillRect(Rect); // clear the rectangle
	MenuAction* a = (MenuAction*)lstActionAtEnd->Items->Objects[Index];
	lstActionAtEnd->Canvas->TextOut(Rect.Left + 4, Rect.Top + 1, (*a));
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::chkSelectionAvailableClick(
	TObject *Sender)
{
	lbPSB->Enabled = chkSelectionAvailable->Checked && rbVideo->Checked;
	edProhibitSelBefore->Enabled = chkSelectionAvailable->Checked && rbVideo->Checked;
	edProhibitSelAfter->Enabled = chkSelectionAvailable->Checked && rbVideo->Checked;

	lbSelOrder->Enabled = chkSelectionAvailable->Checked;
	rbRow->Enabled = chkSelectionAvailable->Checked;
	rbColumn->Enabled = chkSelectionAvailable->Checked;

	lbDefaultSel->Enabled = chkSelectionAvailable->Checked;
	cbDefaultSelection->Enabled = chkSelectionAvailable->Checked;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::PageListDragOver(TObject *Sender,
	TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
  int     idxCurr = PageList->ItemAtPos (Point (X, Y), true);
  Accept = (Source == Sender && idxCurr != -1);

  if (Accept)
  {
    TListBox *lb = (TListBox *) Source;
    int     idxSel = lb->ItemIndex;

    if (idxSel != -1 && idxCurr != idxSel)
    {
      // Swap the 2
      lb->Items->Exchange (idxSel, idxCurr);
      currentPageIndex = idxCurr;
    }
  }
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::ShowPage(int index)
{
	ShowPage(index, true);
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::ShowPage(int index, bool ShowFirstFrame)
{
  previousPageIndex = currentPageIndex;
  currentPageIndex = index;

	if (currentPageIndex != -1)
	{
		Page   *p;

		// Hide previous page
		if (previousPageIndex != -1)
		{
			p = (Page *) PageList->Items->Objects[previousPageIndex];
			p->ShowSelections (false, shapeModel, MainForm->VideoImage);
			currentShape = NULL;
    }
    // Show current page selection
    p = (Page *) PageList->Items->Objects[currentPageIndex];
    p->ShowSelections (true, shapeModel, MainForm->VideoImage);

    // Show all current page data
    edNumber->Text = AnsiString (p->Number);
    edPageName->Text = p->Name;

    if (p->isVideoType)
		{
			if( ShowFirstFrame )
			{
				// Go to video first frame in range
				MainForm->edVideoPos->Text = AnsiString (p->FirstFrameNumber);
				MainForm->bttGoto->Click ();
			}
			rbVideo->Checked = true;
		}
		else
		{
			// Show full size image
			AnsiString filename = MovieDataForm->edBaseDirectory->Text +
					MovieDataForm->edMenuDirectory->Text + "\\" + p->PictureFileName;
			try
			{
				if( !p->PictureFileName.IsEmpty() )
				{
					TJPEGImage* JpegImg = new TJPEGImage();
					JpegImg->LoadFromFile(filename);
					JpegImg->DIBNeeded();
					MainForm->VideoImage->Picture->Bitmap->Assign(JpegImg);
					delete JpegImg;
				}
			} catch(...) { }
			rbStillImage->Checked = true;
    }

    chkSelectionAvailable->Checked = p->SelectionAvailable;
    chkVideoLoop->Checked = p->VideoLoop;

    edPictureFileName->Text = p->PictureFileName;

		edRange->Text = AnsiString (p->FirstFrameNumber) + "-" + p->LastFrameNumber;

    edPictureDuration->Text = AnsiString (p->PictureDuration);

    edProhibitSelBefore->Text = AnsiString (p->ProhibitSelectionBefore);
    edProhibitSelAfter->Text = AnsiString (p->ProhibitSelectionAfter);

		rbColumn->Checked = p->ColumnOrdering;
		rbRow->Checked = !p->ColumnOrdering;		

		if (p->JumpAtEnd != 0)
		{
			// Search index for page number p->JumpAtEnd
			for (int i = 0; i < cbJumpAtEnd->Items->Count; i++)
			{
				if (((Page *) cbJumpAtEnd->Items->Objects[i])->Number == p->JumpAtEnd)
				{
					cbJumpAtEnd->ItemIndex = i;
					break;
				}
			}
		}
		else
		{
			cbJumpAtEnd->ItemIndex = -1;
		}

		lstActionAtEnd->Clear();
		if (p->ActionAtEnd != NULL)
		{
			lstActionAtEnd->Items->AddObject ("", (TObject *) p->ActionAtEnd);
		}

		SelectionsViewerForm->ShowPageSelections(p);

		// Default selection
		UpdateDefaultSelectionCombo(p);
	}
	else
	{
		// Hide previous page
		if (previousPageIndex != -1)
		{
			Page* p = (Page *) PageList->Items->Objects[previousPageIndex];
			p->ShowSelections (false, shapeModel, MainForm->VideoImage);
			currentShape = NULL;
		}

		SelectionsViewerForm->SelectionList->Items->Clear ();
		SelectionsViewerForm->ActionsList->Items->Clear ();
	}

}

//---------------------------------------------------------------------------
void __fastcall TPageViewerForm::UpdateDefaultSelectionCombo(Page* p)
{
	cbDefaultSelection->Items->Clear();
	cbDefaultSelection->Items->Add("None");
	for (vector < Selection * >::iterator i = p->vectSel.begin ();
			 i != p->vectSel.end (); ++i)
	{
		cbDefaultSelection->Items->Add((*i)->Name);
	}
	if( p->DefaultSelection <= 0 || p->DefaultSelection > (cbDefaultSelection->Items->Count-1) )
		cbDefaultSelection->ItemIndex = 0;
	else
		cbDefaultSelection->ItemIndex = p->DefaultSelection;
}
//---------------------------------------------------------------------------
void __fastcall TPageViewerForm::DeletePage(int index)
{
  if (index == -1)
    return;

  // Delete page
  Page   *p = (Page *) PageList->Items->Objects[index];

  PageList->Items->Delete (index);
  delete  p;

  // Update JumpAtEnd combo box
  cbJumpAtEnd->Items->Assign (PageList->Items);

  // Select the page after or the page before if it was the last page
	if (PageList->Items->Count == 0)
	{
		// There is no more page
	}
	else if (index < PageList->Items->Count)
	{
		PageList->ItemIndex = index;
	}
	else
	{
    // We delete the last page
    PageList->ItemIndex = index - 1;
  }

  currentShape = NULL;
  currentPageIndex = -1;
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::PageListContextPopup(TObject *Sender,
	TPoint &MousePos, bool &Handled)
{
	puPLUp->Enabled = ( PageList->ItemIndex > 0 );
	puPLDown->Enabled = ( PageList->ItemIndex != -1 &&
    	(PageList->ItemIndex <  (PageList->Items->Count-1) ) );
	puPLDelete->Enabled = ( PageList->ItemIndex != -1 );
	puPLCopySelections->Enabled = ( PageList->ItemIndex != -1 );
    puPLPasteSelections->Enabled = ( PageList->ItemIndex != -1 &&
	    ClipboardPageIndex != -1 && ClipboardPageIndex != PageList->ItemIndex);

	SetasStartPage1->Enabled = ( PageList->ItemIndex != -1 );
	SetasMenuPage1->Enabled = ( PageList->ItemIndex != -1 );
	SetasChaptersPage1->Enabled = ( PageList->ItemIndex != -1 );
	SetasSpecialsPage1->Enabled = ( PageList->ItemIndex != -1 );
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::puPLCopySelectionsClick(TObject *Sender)
{
	ClipboardPageIndex = PageList->ItemIndex;
	PageList->Refresh();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::puPLPasteSelectionsClick(TObject *Sender)
{
  // Copy selection from page at index ClipboardPageIndex to
  // the page at the current page index

  Page   *pSrc = (Page *) PageList->Items->Objects[ClipboardPageIndex];
  Page   *pDst = (Page *) PageList->Items->Objects[PageList->ItemIndex];

  pSrc->CopySelectionsTo (pDst);

  ShowPage (PageList->ItemIndex);
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::SetasStartPage1Click (TObject * Sender)
{
  if (edNumber->Text == MovieDataForm->edMenuStartPage->Text)
    MovieDataForm->edMenuStartPage->Text = "";
  else
    MovieDataForm->edMenuStartPage->Text = edNumber->Text;
  PageList->Refresh ();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::SetasMenuPage1Click (TObject * Sender)
{
  if (edNumber->Text == MovieDataForm->edMenuMenuPage->Text)
    MovieDataForm->edMenuMenuPage->Text = "";
  else
    MovieDataForm->edMenuMenuPage->Text = edNumber->Text;
  PageList->Refresh ();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::SetasChaptersPage1Click(TObject *Sender)
{
  if (edNumber->Text == MovieDataForm->edMenuChaptersPage->Text)
    MovieDataForm->edMenuChaptersPage->Text = "";
  else
    MovieDataForm->edMenuChaptersPage->Text = edNumber->Text;
  PageList->Refresh ();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::SetasSpecialsPage1Click(TObject *Sender)
{
	if (edNumber->Text == MovieDataForm->edMenuSpecialsPage->Text)
		MovieDataForm->edMenuSpecialsPage->Text = "";
	else
		MovieDataForm->edMenuSpecialsPage->Text = edNumber->Text;
	PageList->Refresh ();
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::ReInit()
{
	// PageList
	for(int i = 0; i < PageViewerForm->PageList->Items->Count; i++)
	{
		delete (Page*)PageViewerForm->PageList->Items->Objects[i];
	}
	PageViewerForm->PageList->Clear();

	edNumber->Text = "";
	edPageName->Text = "";
	rbVideo->Checked = true;
	edPictureFileName->Text = "";
	chkSelectionAvailable->Checked = true;
	chkVideoLoop->Checked = true;
	edRange->Text ="0-0";
	edPictureDuration->Text = 0;
	cbJumpAtEnd->Clear();
	lstActionAtEnd->Clear();
	edProhibitSelBefore->Text = "0";
	edProhibitSelAfter->Text = "0";
	cbDefaultSelection->Clear();
	previousPageIndex = -1;
	currentPageIndex = -1;	
}

//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::lstActionAtEndKeyDown(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
	// delete action
	if( Key == VK_DELETE)
	{
		lstActionAtEnd->Clear();
		lstActionAtEnd->Refresh();

		if(PageList->ItemIndex != -1)
		{
			Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];
			if(p->ActionAtEnd != NULL)
			{
				delete p->ActionAtEnd;
				p->ActionAtEnd = NULL;
			}
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TPageViewerForm::lstActionAtEndDblClick(TObject *Sender)
{
	lstActionAtEndDragDrop(SelectionsViewerForm->DragDropActionPanel,lstActionAtEnd,0,0);
}
//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::miSetActionAtEndClick(TObject *Sender)
{
	lstActionAtEndDragDrop(SelectionsViewerForm->DragDropActionPanel,lstActionAtEnd,0,0);
}
//---------------------------------------------------------------------------

void __fastcall TPageViewerForm::miDeleteActionAtEndClick(TObject *Sender)
{
	if(PageList->ItemIndex == -1)
		return;

	Page* p = (Page*)PageList->Items->Objects[PageList->ItemIndex];

	lstActionAtEnd->Clear();
	lstActionAtEnd->Refresh();

	if(p->ActionAtEnd != NULL)
	{
		delete p->ActionAtEnd;
		p->ActionAtEnd = NULL;
	}
}
//---------------------------------------------------------------------------

