/***************************************************************************\
 * Micro DVD Menu Maker by Christophe.Paris@free.fr                        *
 ***************************************************************************
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
\***************************************************************************/

//---------------------------------------------------------------------------
#ifndef _RSHAPE_H_
#define _RSHAPE_H_
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <ExtCtrls.hpp>
//---------------------------------------------------------------------------
#define ANCHOR_SIZE 6
#define ANCHOR_BG_COLOR clWhite

typedef void __fastcall (__closure * TRShapeChangedSizeEvent) (void);
typedef void __fastcall (__closure * TRShapeAnyAchorMouseMove) (void);
//---------------------------------------------------------------------------
class RShape : public TShape
{
public:
	virtual __fastcall RShape (Classes::TComponent * AOwner):TShape (AOwner)
	{
		rshapeAOwner = AOwner;
		anchorsAreShown = false;
	}

  void __fastcall CopyProperties (TShape * src, TShape * dst)
  {
    dst->Width = src->Width;
    dst->Height = src->Height;
    dst->Brush->Color = src->Brush->Color;
    dst->OnMouseDown = src->OnMouseDown;
    dst->OnMouseUp = src->OnMouseUp;
    dst->Parent = src->Parent;
  }

  virtual __fastcall ~ RShape ()
  {
    // Shape are deleted by parent
  }


  void __fastcall CreateAnchors (Classes::TComponent * AOwner)
  {
    // Create anchors
    leftShape = new TShape (this);
    rightShape = new TShape (this);
    topShape = new TShape (this);
    bottomShape = new TShape (this);
    leftTopShape = new TShape (this);
    leftBottShape = new TShape (this);
    rightTopShape = new TShape (this);
    rightBottShape = new TShape (this);

    // Set all common properties for one anchor
    leftShape->Width = ANCHOR_SIZE;
    leftShape->Height = ANCHOR_SIZE;
    leftShape->Brush->Color = ANCHOR_BG_COLOR;
    leftShape->OnMouseDown = ShapeMouseDown;
    leftShape->OnMouseUp = ShapeMouseUp;
    leftShape->Parent = dynamic_cast < TWinControl * >(AOwner);

    // Copy common properties for every other anchors
    CopyProperties (leftShape, rightShape);
    CopyProperties (leftShape, topShape);
    CopyProperties (leftShape, bottomShape);
    CopyProperties (leftShape, leftTopShape);
    CopyProperties (leftShape, leftBottShape);
    CopyProperties (leftShape, rightTopShape);
    CopyProperties (leftShape, rightBottShape);

    // Set cursor when on anchor
    leftShape->Cursor = crSizeWE;
    rightShape->Cursor = crSizeWE;
    topShape->Cursor = crSizeNS;
    bottomShape->Cursor = crSizeNS;
    leftTopShape->Cursor = crSizeNWSE;
    leftBottShape->Cursor = crSizeNESW;
    rightTopShape->Cursor = crSizeNESW;
    rightBottShape->Cursor = crSizeNWSE;

    // Set OnMouseMove Event
    leftShape->OnMouseMove = ShapeLMouseMove;
    rightShape->OnMouseMove = ShapeRMouseMove;
    topShape->OnMouseMove = ShapeTMouseMove;
    bottomShape->OnMouseMove = ShapeBMouseMove;
    leftTopShape->OnMouseMove = ShapeLTMouseMove;
    leftBottShape->OnMouseMove = ShapeLBMouseMove;
    rightTopShape->OnMouseMove = ShapeRTMouseMove;
    rightBottShape->OnMouseMove = ShapeRBMouseMove;
  }

  void __fastcall ShowAnchors (bool Show)
  {
    if (anchorsAreShown)
    {
      if (!Show)
      {
				delete  leftShape;
				delete  rightShape;
				delete  topShape;
				delete  bottomShape;
				delete  leftTopShape;
				delete  leftBottShape;
				delete  rightTopShape;
				delete  rightBottShape;
				anchorsAreShown = false;
			}
		}
		else
		{
			if (Show)
			{
				CreateAnchors (rshapeAOwner);
				// ReAdjust position
				ReAdjustAnchors ();
				// Bring Anchors to front
				leftShape->BringToFront ();
				rightShape->BringToFront ();
				topShape->BringToFront ();
				bottomShape->BringToFront ();
				leftTopShape->BringToFront ();
				leftBottShape->BringToFront ();
				rightTopShape->BringToFront ();
				rightBottShape->BringToFront ();
				anchorsAreShown = true;
			}
		}
	}

	void __fastcall ReAdjustAnchors ()
	{
    leftShape->Left = Left;
    leftShape->Top = Top + (Height - ANCHOR_SIZE) / 2;
    rightShape->Left = Left + Width - ANCHOR_SIZE;
    rightShape->Top = Top + (Height - ANCHOR_SIZE) / 2;
    topShape->Left = Left + (Width - ANCHOR_SIZE) / 2;
    topShape->Top = Top;
    bottomShape->Left = Left + (Width - ANCHOR_SIZE) / 2;
    bottomShape->Top = Top + Height - ANCHOR_SIZE;

    leftTopShape->Left = Left;
    leftTopShape->Top = Top;
    leftBottShape->Left = Left;
    leftBottShape->Top = Top + Height - ANCHOR_SIZE;
    rightTopShape->Left = Left + Width - ANCHOR_SIZE;
    rightTopShape->Top = Top;
    rightBottShape->Left = Left + Width - ANCHOR_SIZE;
    rightBottShape->Top = Top + Height - ANCHOR_SIZE;
  }

private:
  // Anchors mouse events
  void __fastcall ShapeMouseDown (TObject * Sender, TMouseButton Button,
					TShiftState Shift, int X, int Y)
  {
    if (Button == mbLeft)
    {
      origX = X;
      origY = Y;
    }
  }

  void __fastcall ShapeMouseUp (TObject * Sender, TMouseButton Button,
				TShiftState Shift, int X, int Y)
  {
    if (Button == mbLeft)
    {
      ReAdjustAnchors ();
      if (FOnChangedSize != NULL)
				FOnChangedSize ();
    }
  }

  void __fastcall ShapeLMouseMove (TObject * Sender, TShiftState Shift, int X,
				   int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      leftShape->Left += (X - origX);
      Left += (X - origX);
      Width = (rightShape->Left + ANCHOR_SIZE) - Left;
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeRMouseMove (TObject * Sender, TShiftState Shift, int X,
				   int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      rightShape->Left += (X - origX);
      Width += (X - origX);
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeTMouseMove (TObject * Sender, TShiftState Shift, int X,
				   int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      topShape->Top += (Y - origY);
      Top += (Y - origY);
      Height = (bottomShape->Top + ANCHOR_SIZE) - Top;
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeBMouseMove (TObject * Sender, TShiftState Shift, int X,
				   int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      bottomShape->Top += (Y - origY);
      Height += (Y - origY);
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeLTMouseMove (TObject * Sender, TShiftState Shift, int X,
				    int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      leftTopShape->Left += (X - origX);
      leftTopShape->Top += (Y - origY);
      Top += (Y - origY);
      Height = (bottomShape->Top + ANCHOR_SIZE) - Top;
      Left += (X - origX);
      Width = (rightShape->Left + ANCHOR_SIZE) - Left;
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeLBMouseMove (TObject * Sender, TShiftState Shift, int X,
				    int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      leftBottShape->Left += (X - origX);
      leftBottShape->Top += (Y - origY);
      Height += (Y - origY);
      Left += (X - origX);
      Width = (rightShape->Left + ANCHOR_SIZE) - Left;
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeRTMouseMove (TObject * Sender, TShiftState Shift, int X,
						int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();


    if (Shift.Contains (ssLeft))
    {
      rightTopShape->Left += (X - origX);
      rightTopShape->Top += (Y - origY);
      Top += (Y - origY);
      Height = (bottomShape->Top + ANCHOR_SIZE) - Top;
      Width += (X - origX);
      ReAdjustAnchors ();
    }
  }

  void __fastcall ShapeRBMouseMove (TObject * Sender, TShiftState Shift, int X,
				    int Y)
  {

    if (FOnAnyAnchorMove != NULL)
      FOnAnyAnchorMove ();

    if (Shift.Contains (ssLeft))
    {
      rightBottShape->Left += (X - origX);
      rightBottShape->Top += (Y - origY);
      Height += (Y - origY);
      Width += (X - origX);
      ReAdjustAnchors ();
    }
  }

  bool __fastcall GetVisible ()
  {
    return TShape::Visible;
  }

  void __fastcall SetVisible (bool vis)
  {
    TShape::Visible = vis;
    if (!vis)
    {
      ShowAnchors (false);
    }
  }

__published:
	__property Visible = { read = GetVisible, write = SetVisible, default = 1};
	__property TRShapeChangedSizeEvent OnChangedSize = { read = FOnChangedSize, write = FOnChangedSize };
	__property TRShapeAnyAchorMouseMove OnAnyAnchorMove = { read = FOnAnyAnchorMove, write = FOnAnyAnchorMove };


private:
	int     origX, origY;		// Original coordinate when moving shape

	TShape *leftShape;
	TShape *rightShape;
	TShape *topShape;
	TShape *bottomShape;

	TShape *leftTopShape;
	TShape *leftBottShape;
	TShape *rightTopShape;
	TShape *rightBottShape;

	TRShapeChangedSizeEvent FOnChangedSize;
	TRShapeAnyAchorMouseMove FOnAnyAnchorMove;

	bool    anchorsAreShown;
	Classes::TComponent * rshapeAOwner;
};

//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
