/***************************************************************************\
 * Micro DVD Menu Maker by Christophe.Paris@free.fr                        *
 ***************************************************************************
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
\***************************************************************************/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "SelectionViewer.h"
#include "PageViewer.h"
#include "MovieData.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSelectionsViewerForm *SelectionsViewerForm;


//---------------------------------------------------------------------------

__fastcall TSelectionsViewerForm::TSelectionsViewerForm(TComponent* Owner)
	: TForm(Owner)
{
	Top  = MainForm->Top;
	Left = PageViewerForm->Left +	PageViewerForm->Width;
	
  for (int i = 0; i < ACTIONS_NUMBER; i++)
  {
    cbAction->Items->Add (ActionsListDef[i].name);
  }
	cbAction->ItemIndex = 0;
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ShowPageSelections(Page* p)
{
  // Clean up
  SelectionList->Items->Clear ();
  ActionsList->Items->Clear ();

  TListItem *li;
  for (vector < Selection * >::iterator i = p->vectSel.begin ();
       i != p->vectSel.end (); ++i)
  {
    li = SelectionList->Items->Add ();
    li->Caption = (*i)->Name;
    li->Data = (*i);
	}
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::SelectSelection(AnsiString Name) {
	TListItem* li = SelectionList->FindCaption(0,Name,false,true,false);
	SelectionList->Selected = li;

	SelectionListClick(this);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::SelectSelection(Selection *s) {
	TListItem* li = SelectionList->FindData(0, s, true, false);
	SelectionList->Selected = li;

	SelectionListClick(this);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::SelectionListClick(TObject *Sender)
{
	if (SelectionList->Selected != NULL)
  {
    Selection *s = (Selection *) SelectionList->Selected->Data;
    MainForm->ChangeCurrentSelectedShape (s->shape);

    // Update Action list
    ActionsList->Items->Clear ();
    for (vector < MenuAction * >::iterator i = s->vectAction.begin ();
			i != s->vectAction.end (); ++i)
    {
			ActionsList->Items->AddObject ("", (TObject *) (*i));
		}
		// Select first action & show details about it		
		if(ActionsList->Items->Count > 0) {
			ActionsList->ItemIndex = 0;
			ActionsListClick(this);
		}
  }
}


//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ActionsListDragOver(TObject *Sender,
	TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
	// Check source of drag & accept
	Accept = ( Source == dynamic_cast<TObject *>(DragDropActionPanel) ) &&
		(SelectionList->Selected != NULL);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ActionsListDragDrop(TObject *Sender,
	TObject *Source, int X, int Y)
{
	Selection* s = (Selection*)SelectionList->Selected->Data;
	if( s == NULL) {
		return;
	}

	MenuAction* a = GetDragedAction();

	s->vectAction.push_back( a );

	ActionsList->Items->AddObject("",(TObject*)a);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::cbActionChange(TObject *Sender)
{
	if(cbAction->ItemIndex == -1)
			return;

	Page* nextP = NULL, *prevP = NULL;			
	AnsiString nextPName, prevPName;

	switch(cbAction->ItemIndex)
	{
		case JUMP_TO_NEW_PAGE:
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csOwnerDrawFixed;
				cbActionParameter->Hint = "Choose a page";
				// Copy page name from page
				cbActionParameter->Items->Assign(PageViewerForm->PageList->Items);
				cbActionParameter->ItemIndex = 0;
				break;
		case PLAY_MOVIE:
				cbActionParameter->Hint = "No parameters";
				cbActionParameter->Enabled = false;
				cbActionParameter->Items->Clear();
				break;
		case PLAY_CHAPTER:
				cbActionParameter->Hint = "Enter chapter number";
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csDropDownList;
				cbActionParameter->Items->Clear();
				// Look if we have full chapters info
				if( MovieDataForm->lstChapters->Items->Count > 0 ) {
					for(int i = 0; i < MovieDataForm->lstChapters->Items->Count; i++)
					{
						cbActionParameter->Items->Add( AnsiString(i+1) + ") " +
							MovieDataForm->lstChapters->Items->Item[i]->SubItems->Strings[1]);
					}
				}
				else
				{
					// Just add 50 "empty" chapters
					for(int i=1; i < 50; i++)
						cbActionParameter->Items->Add(i);
				}
				cbActionParameter->ItemIndex = 0;
				break;
		case DISPLAY_MSGBOX:
				cbActionParameter->Hint = "Enter message";
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csSimple;
				cbActionParameter->Items->Clear();
				cbActionParameter->Text = "";
				break;
		case PLAY_SPECIAL_FEATURE_VIDEO:
				cbActionParameter->Hint = "Enter special video feature number";
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csDropDownList;
				cbActionParameter->Items->Clear();
				// Look if we have full specials infos
				if( MovieDataForm->lstSpecials->Items->Count > 0 )
				{
					for(int i = 0; i < MovieDataForm->lstSpecials->Items->Count; i++)
					{
						cbActionParameter->Items->Add( AnsiString(i+1) + ") " +
							MovieDataForm->lstSpecials->Items->Item[i]->SubItems->Strings[0]);
					}
				}
				else
				{
					// Just add 50 "empty" specials
					for(int i=1; i < 50; i++)
						cbActionParameter->Items->Add(i);
				}
				cbActionParameter->ItemIndex = 0;
				break;
		case CHANGE_LANGUAGE:
				cbActionParameter->Hint = "Enter language number";
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csDropDownList;
				cbActionParameter->Items->Clear();
				// Look if we have full language infos
				if( MovieDataForm->lstLanguages->Items->Count > 0 )
				{
					for(int i = 0; i < MovieDataForm->lstLanguages->Items->Count; i++)
					{
						cbActionParameter->Items->Add( AnsiString(i) + ") " +
							MovieDataForm->lstLanguages->Items->Item[i]->SubItems->Strings[1]);
					}
				}
				else
				{
					// Just add 50 "empty" language
					for(int i=0; i < 6; i++)
						cbActionParameter->Items->Add(i);
				}
				cbActionParameter->ItemIndex = 0;
				break;
		case CHANGE_SUBTITLE:
				cbActionParameter->Hint = "Enter subtitle number";
				cbActionParameter->Enabled = true;
				cbActionParameter->Style = csDropDownList;
				cbActionParameter->Items->Clear();

				// Look if we have full language infos
				if( MovieDataForm->lstSubtitles->Items->Count > 0 ) {
					cbActionParameter->Items->Add("0) No subtitle");
					for(int i = 0; i < MovieDataForm->lstSubtitles->Items->Count; i++)
					{
						cbActionParameter->Items->Add( AnsiString(i+1) + ") " +
							MovieDataForm->lstSubtitles->Items->Item[i]->SubItems->Strings[1]);
					}
				}
				else
				{
					// Just add 10 "empty" subtitles
					for(int i=0; i < 10; i++)
						cbActionParameter->Items->Add(i);
				}
				cbActionParameter->ItemIndex = 0;
				break;
		case RETURN_TO_MOVIE:
				cbActionParameter->Hint = "No parameters";
				cbActionParameter->Enabled = false;
				cbActionParameter->Items->Clear();
				break;
		case RELATIVE_JUMP:
				cbActionParameter->Hint = "Jump to next page or previous page.";
				cbActionParameter->Style = csDropDownList;
				cbActionParameter->Items->Clear();
				cbActionParameter->Enabled = true;
				// TODO : add the page name if it exist
				if(PageViewerForm->PageList->ItemIndex+1 < PageViewerForm->PageList->Items->Count)
					nextP = (Page*)PageViewerForm->PageList->Items->Objects[PageViewerForm->PageList->ItemIndex+1];
				if(PageViewerForm->PageList->ItemIndex - 1 >= 0)
					prevP = (Page*)PageViewerForm->PageList->Items->Objects[PageViewerForm->PageList->ItemIndex-1];

				nextPName = (nextP ? nextP->Name : AnsiString("NULL"));
				prevPName = (prevP ? prevP->Name : AnsiString("NULL"));

				cbActionParameter->Items->Add("+1) Next page (" + nextPName + ")");
				cbActionParameter->Items->Add("-1) Previous page (" + prevPName + ")");
				cbActionParameter->ItemIndex = 0;				
				break;
		default:
				ShowMessage("Undefined Action !");
				break;
		}
	cbAction->Hint = ActionsListDef[cbAction->ItemIndex].helpMsg;		
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::FormActivate(TObject *Sender)
{
	// Update the action maker
	cbActionChange(this);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::bttActionMakerHelpClick(
      TObject *Sender)
{
	MessageBox(Handle, ActionsListDef[cbAction->ItemIndex].helpMsg,"Help",MB_OK);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ActionsListDrawItem(
      TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
  ActionsList->Canvas->FillRect (Rect);	// clear the rectangle
  MenuAction *a = (MenuAction *) ActionsList->Items->Objects[Index];
  ActionsList->Canvas->TextOut (Rect.Left + 4, Rect.Top, (*a));
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::Delete1Click(TObject *Sender)
{
  if (ActionsList->ItemIndex == -1)
    return;

  // Delete selected action
  MenuAction *a =
    (MenuAction *) ActionsList->Items->Objects[ActionsList->ItemIndex];
  ActionsList->Items->Delete (ActionsList->ItemIndex);

  Selection *s = (Selection *) SelectionList->Selected->Data;

  vector < MenuAction * >::iterator ai =
    find (s->vectAction.begin (), s->vectAction.end (), a);
  s->vectAction.erase (ai);
  delete  a;
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::UpDownClick(TObject *Sender)
{
	TMenuItem *mi = (TMenuItem*)Sender;
	int UpDownOffset = mi->Tag;
	int newIndex = ActionsList->ItemIndex + UpDownOffset;

	//Check if the newIndex is bounded
	if( newIndex < 0 || newIndex > (ActionsList->Items->Count - 1) ) {
		return;
	}

	// Exchange from selection action vector
	MenuAction* a1 = (MenuAction*)ActionsList->Items->Objects[ActionsList->ItemIndex];
	MenuAction* a2 = (MenuAction*)ActionsList->Items->Objects[newIndex];

	Selection* s = (Selection*)SelectionList->Selected->Data;
	vector<MenuAction*>::iterator ai1 = find(s->vectAction.begin(), s->vectAction.end(), a1);
	vector<MenuAction*>::iterator ai2 = find(s->vectAction.begin(), s->vectAction.end(), a2);
	(*ai1) = a2;
	(*ai2) = a1;

	// Exchange from display
	ActionsList->Items->Exchange(ActionsList->ItemIndex, newIndex);

}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::SelectionListKeyUp(TObject *Sender,
	WORD &Key, TShiftState Shift)
{
  if (Key == VK_UP || Key == VK_DOWN)
  {
    SelectionListClick (this);
  }
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::cbActionParameterDrawItem(
	TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
	cbActionParameter->Canvas->FillRect(Rect); // clear the rectangle
	Page* p = (Page*)cbActionParameter->Items->Objects[Index];
	cbActionParameter->Canvas->TextOut(Rect.Left + 2, Rect.Top + 1 ,
		p->GetShortName());
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ActionsListClick(TObject *Sender)
{
	if( ActionsList->ItemIndex == -1 )
			return;

	MenuAction* ma = (MenuAction*)ActionsList->Items->Objects[ActionsList->ItemIndex];
	cbAction->ItemIndex = ma->type - 1;
	// Update parameter list
	cbAction->OnChange(this);

	// Show parameter
	switch(cbAction->ItemIndex)
	{
		case PLAY_MOVIE:
		case RETURN_TO_MOVIE:
			cbActionParameter->Text = ma->param;
			break;
		case JUMP_TO_NEW_PAGE:
		case PLAY_CHAPTER:
		case PLAY_SPECIAL_FEATURE_VIDEO:
		case CHANGE_LANGUAGE:
		case CHANGE_SUBTITLE:
		case RELATIVE_JUMP:
				// search the right parameter
				for( int i = 0; i < cbActionParameter->Items->Count; i++)
				{
					if( ma->param == cbActionParameter->Items->Strings[i].SubString(1,ma->param.Length()) )
					{
						cbActionParameter->ItemIndex = i;
						break;
					}
				}
			break;
		case DISPLAY_MSGBOX:
			cbActionParameter->Text = ma->param;
			break;
		default:
			cbActionParameter->Text = "";
	}

}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ActionsListContextPopup(
	TObject *Sender, TPoint &MousePos, bool &Handled)
{
	Up1->Enabled = ( ActionsList->ItemIndex > 0 );
	Down1->Enabled = ( ActionsList->ItemIndex != -1 &&
		(ActionsList->ItemIndex <  (ActionsList->Items->Count-1) ) );
	Delete1->Enabled = ( ActionsList->ItemIndex != -1 );
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::ReInit()
{
	SelectionList->Items->Clear();
	ActionsList->Clear();
	cbAction->ItemIndex = 0;
	cbActionParameter->Clear();
}

//---------------------------------------------------------------------------

MenuAction* __fastcall TSelectionsViewerForm::GetDragedAction()
{
	int number = cbAction->ItemIndex+1;
	AnsiString parameter = "";

	switch(cbAction->ItemIndex) {
		case PLAY_MOVIE:
		case RETURN_TO_MOVIE:
			parameter = "";
			break;
		case JUMP_TO_NEW_PAGE:
		case PLAY_CHAPTER:
		case PLAY_SPECIAL_FEATURE_VIDEO:
		case CHANGE_LANGUAGE:
		case CHANGE_SUBTITLE:
		case RELATIVE_JUMP:		
			{
				int pos = cbActionParameter->Text.Pos(")");
				if( pos == 0 ) {
					parameter = cbActionParameter->Text;
				} else {
					parameter = cbActionParameter->Text.SubString(1,pos-1);
				}
				// Auto increment parameter for next action
				cbActionParameter->ItemIndex += 1;
			}
			break;
		case DISPLAY_MSGBOX:
			parameter = cbActionParameter->Text;
			break;
		default:
			parameter = "";
	}

	return new MenuAction(number, parameter);
}

//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::bttSelAddActionClick(
			TObject *Sender)
{
	if(SelectionList->Selected == NULL)
		return;

	Selection* s = (Selection*)SelectionList->Selected->Data;
	if( s == NULL)
		return;

	MenuAction* a = GetDragedAction();
	s->vectAction.push_back( a );
	ActionsList->Items->AddObject("",(TObject*)a);
}
//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::SelectionListEdited(TObject *Sender,
      TListItem *Item, AnsiString &S)
{
	Selection *s = (Selection *) Item->Data;
	if (s != NULL)
		s->Name = S;

	Page* p = (Page*)PageViewerForm->PageList->Items->Objects[PageViewerForm->PageList->ItemIndex];
	PageViewerForm->UpdateDefaultSelectionCombo(p);
}
//---------------------------------------------------------------------------

void __fastcall TSelectionsViewerForm::bttModifyCurrentActionClick(
      TObject *Sender)
{
	if(ActionsList->ItemIndex != -1)
	{
		Selection* s = (Selection*)SelectionList->Selected->Data;
		MenuAction* maOld = s->vectAction[ActionsList->ItemIndex];
		MenuAction* maNew = GetDragedAction();
		s->vectAction[ActionsList->ItemIndex] = maNew;
		ActionsList->Items->Objects[ActionsList->ItemIndex] = (TObject*)maNew;
		delete maOld;
		ActionsList->Refresh();
	}
}
//---------------------------------------------------------------------------

